"""
Interface du repository des rôles.
"""

from abc import ABC, abstractmethod
from typing import Optional, List

from alak_acl.roles.domain.entities.role import Role



class IRoleRepository(ABC):
    """
    Interface abstraite pour le repository des rôles.

    Définit le contrat que chaque implémentation de repository
    (MongoDB, PostgreSQL, MySQL) doit respecter.
    """

    # ==========================================
    # CRUD Rôles
    # ==========================================

    @abstractmethod
    async def create_role(self, role: Role) -> Role:
        """
        Crée un nouveau rôle.

        Args:
            role: Entité rôle à créer

        Returns:
            Rôle créé avec son ID

        Raises:
            RoleAlreadyExistsError: Si un rôle avec ce nom existe déjà
        """
        pass

    @abstractmethod
    async def get_by_id(self, role_id: str) -> Optional[Role]:
        """
        Récupère un rôle par son ID.

        Args:
            role_id: ID du rôle

        Returns:
            Rôle ou None si non trouvé
        """
        pass

    @abstractmethod
    async def get_by_name(self, name: str) -> Optional[Role]:
        """
        Récupère un rôle par son nom.

        Args:
            name: Nom du rôle

        Returns:
            Rôle ou None si non trouvé
        """
        pass

    @abstractmethod
    async def update_role(self, role: Role) -> Role:
        """
        Met à jour un rôle existant.

        Args:
            role: Entité rôle avec les modifications

        Returns:
            Rôle mis à jour

        Raises:
            RoleNotFoundError: Si le rôle n'existe pas
        """
        pass

    @abstractmethod
    async def delete_role(self, role_id: str) -> bool:
        """
        Supprime un rôle.

        Args:
            role_id: ID du rôle

        Returns:
            True si supprimé, False si non trouvé

        Raises:
            PermissionDeniedError: Si le rôle est un rôle système
        """
        pass

    @abstractmethod
    async def list_roles(
        self,
        skip: int = 0,
        limit: int = 100,
        is_active: Optional[bool] = None,
        tenant_id: Optional[str] = None,
    ) -> List[Role]:
        """
        Liste les rôles avec pagination.

        Args:
            skip: Nombre d'éléments à sauter
            limit: Nombre maximum d'éléments
            is_active: Filtrer par statut actif (optionnel)
            tenant_id: Filtrer par tenant (optionnel)

        Returns:
            Liste des rôles
        """
        pass

    @abstractmethod
    async def count_roles(
        self,
        is_active: Optional[bool] = None,
        tenant_id: Optional[str] = None,
    ) -> int:
        """
        Compte le nombre de rôles.

        Args:
            is_active: Filtrer par statut actif (optionnel)
            tenant_id: Filtrer par tenant (optionnel)

        Returns:
            Nombre de rôles
        """
        pass

    @abstractmethod
    async def role_exists(self, name: str) -> bool:
        """
        Vérifie si un rôle existe par son nom.

        Args:
            name: Nom du rôle

        Returns:
            True si existe
        """
        pass

    @abstractmethod
    async def get_default_roles(self) -> List[Role]:
        """
        Récupère tous les rôles par défaut.

        Returns:
            Liste des rôles marqués comme default
        """
        pass

    @abstractmethod
    async def get_default_role(self) -> Optional[Role]:
        """
        Récupère le rôle par défaut (premier rôle marqué is_default=True).

        Returns:
            Rôle par défaut ou None
        """
        pass

    # ==========================================
    # Memberships (user <-> tenant <-> role)
    # ==========================================

    @abstractmethod
    async def assign_role_to_user(
        self,
        user_id: str,
        role_id: str,
        tenant_id: Optional[str] = None,
        assigned_by: Optional[str] = None,
    ) -> bool:
        """
        Assigne un rôle à un utilisateur dans un tenant (crée un membership).

        Args:
            user_id: ID de l'utilisateur
            role_id: ID du rôle
            tenant_id: ID du tenant (None pour rôle global sans tenant)
            assigned_by: ID de l'utilisateur ayant fait l'assignation (optionnel)

        Returns:
            True si assigné avec succès

        Raises:
            RoleNotFoundError: Si le rôle n'existe pas
        """
        pass

    @abstractmethod
    async def remove_role_from_user(
        self,
        user_id: str,
        role_id: str,
        tenant_id: str,
    ) -> bool:
        """
        Retire un rôle d'un utilisateur dans un tenant.

        Args:
            user_id: ID de l'utilisateur
            role_id: ID du rôle
            tenant_id: ID du tenant

        Returns:
            True si retiré avec succès
        """
        pass

    @abstractmethod
    async def get_user_roles(
        self,
        user_id: str,
        tenant_id: Optional[str] = None,
    ) -> List[Role]:
        """
        Récupère les rôles d'un utilisateur.

        Args:
            user_id: ID de l'utilisateur
            tenant_id: ID du tenant (si None, retourne les rôles de tous les tenants)

        Returns:
            Liste des rôles de l'utilisateur
        """
        pass

    @abstractmethod
    async def get_user_tenants(self, user_id: str) -> List[str]:
        """
        Récupère les IDs des tenants auxquels un utilisateur appartient.

        Args:
            user_id: ID de l'utilisateur

        Returns:
            Liste des IDs de tenants
        """
        pass

    @abstractmethod
    async def get_users_with_role(
        self,
        role_id: str,
        tenant_id: Optional[str] = None,
        skip: int = 0,
        limit: int = 100,
    ) -> List[str]:
        """
        Récupère les IDs des utilisateurs ayant un rôle spécifique.

        Args:
            role_id: ID du rôle
            tenant_id: ID du tenant (si None, cherche dans tous les tenants)
            skip: Offset pour la pagination
            limit: Limite du nombre de résultats

        Returns:
            Liste des IDs utilisateurs
        """
        pass

    @abstractmethod
    async def user_has_role(
        self,
        user_id: str,
        role_id: str,
        tenant_id: str,
    ) -> bool:
        """
        Vérifie si un utilisateur a un rôle dans un tenant.

        Args:
            user_id: ID de l'utilisateur
            role_id: ID du rôle
            tenant_id: ID du tenant

        Returns:
            True si l'utilisateur a le rôle dans ce tenant
        """
        pass

    @abstractmethod
    async def user_has_role_by_name(
        self,
        user_id: str,
        role_name: str,
        tenant_id: str,
    ) -> bool:
        """
        Vérifie si un utilisateur a un rôle par son nom dans un tenant.

        Args:
            user_id: ID de l'utilisateur
            role_name: Nom du rôle
            tenant_id: ID du tenant

        Returns:
            True si l'utilisateur a le rôle dans ce tenant
        """
        pass

    @abstractmethod
    async def get_user_permissions(
        self,
        user_id: str,
        tenant_id: str,
    ) -> List[str]:
        """
        Récupère toutes les permissions d'un utilisateur dans un tenant.

        Args:
            user_id: ID de l'utilisateur
            tenant_id: ID du tenant

        Returns:
            Liste des permissions uniques
        """
        pass

    @abstractmethod
    async def set_user_roles(
        self,
        user_id: str,
        role_ids: List[str],
        tenant_id: str,
    ) -> bool:
        """
        Définit la liste complète des rôles d'un utilisateur dans un tenant.

        Remplace tous les rôles existants dans ce tenant.

        Args:
            user_id: ID de l'utilisateur
            role_ids: Liste des IDs de rôles
            tenant_id: ID du tenant

        Returns:
            True si succès
        """
        pass

    @abstractmethod
    async def clear_user_roles(
        self,
        user_id: str,
        tenant_id: Optional[str] = None,
    ) -> bool:
        """
        Supprime les rôles d'un utilisateur.

        Args:
            user_id: ID de l'utilisateur
            tenant_id: ID du tenant (si None, supprime dans tous les tenants)

        Returns:
            True si succès
        """
        pass

    @abstractmethod
    async def get_tenant_members(
        self,
        tenant_id: str,
        skip: int = 0,
        limit: int = 100,
    ) -> List[str]:
        """
        Récupère les IDs des utilisateurs membres d'un tenant.

        Args:
            tenant_id: ID du tenant
            skip: Offset pour la pagination
            limit: Limite du nombre de résultats

        Returns:
            Liste des IDs utilisateurs
        """
        pass

    @abstractmethod
    async def count_tenant_members(self, tenant_id: str) -> int:
        """
        Compte le nombre de membres d'un tenant.

        Args:
            tenant_id: ID du tenant

        Returns:
            Nombre de membres
        """
        pass

    # ==========================================
    # Vérifications pour la suppression
    # ==========================================

    @abstractmethod
    async def count_roles_with_permission(self, permission_name: str) -> int:
        """
        Compte le nombre de rôles qui utilisent une permission.

        Args:
            permission_name: Nom de la permission (ex: "posts:create")

        Returns:
            Nombre de rôles utilisant cette permission
        """
        pass



"""
Modèle Pydantic pour les permissions MongoDB.

Utilisé pour la validation et sérialisation des documents MongoDB.
"""

from datetime import datetime
from typing import Optional

from pydantic import BaseModel, Field


class MongoPermissionModel(BaseModel):
    """
    Modèle Pydantic pour les permissions MongoDB.

    Attributes:
        id: ID du document (alias pour _id MongoDB)
        resource: Ressource concernée
        action: Action autorisée
        name: Nom complet (resource:action)
        display_name: Nom d'affichage
        description: Description
        category: Catégorie pour regroupement
        is_active: Permission active
        is_system: Permission système
        created_at: Date de création
        updated_at: Date de mise à jour
    """

    id: Optional[str] = Field(default=None, alias="_id")
    resource: str = Field(..., max_length=50)
    action: str = Field(..., max_length=50)
    name: str = Field(..., max_length=101)
    display_name: Optional[str] = Field(default=None, max_length=100)
    description: Optional[str] = Field(default=None)
    category: Optional[str] = Field(default=None, max_length=50)
    is_active: bool = Field(default=True)
    is_system: bool = Field(default=False)
    created_at: datetime = Field(default_factory=datetime.utcnow)
    updated_at: Optional[datetime] = Field(default=None)

    model_config = {
        "populate_by_name": True,
        "json_encoders": {
            datetime: lambda v: v.isoformat() if v else None,
        },
    }

    def to_mongo_dict(self, include_id: bool = False) -> dict:
        """
        Convertit le modèle en dictionnaire pour MongoDB.

        Args:
            include_id: Inclure l'_id dans le dictionnaire

        Returns:
            Dictionnaire pour insertion/mise à jour MongoDB
        """
        data = self.model_dump(by_alias=True, exclude_none=False)

        # Ne pas inclure _id si None ou si non demandé
        if not include_id or data.get("_id") is None:
            data.pop("_id", None)

        return data

    @classmethod
    def from_mongo_doc(cls, doc: dict) -> "MongoPermissionModel":
        """
        Crée une instance depuis un document MongoDB.

        Args:
            doc: Document MongoDB

        Returns:
            Instance du modèle
        """
        if doc is None:
            return None

        # Convertir ObjectId en string
        if "_id" in doc:
            doc["_id"] = str(doc["_id"])

        return cls(**doc)



"""
Feature Permissions - Gestion des permissions.

Cette feature gère les permissions sous forme resource:action.
Supporte les wildcards pour les permissions globales.

Example:
    ```python
    from fastapi import FastAPI
    from alak_acl import ACLManager, ACLConfig
    from alak_acl.permissions import Permission, CreatePermissionDTO

    app = FastAPI()

    config = ACLConfig(
        database_type="postgresql",
        postgresql_uri="postgresql+asyncpg://user:pass@localhost/db",
        enable_permissions_feature=True,
    )

    acl = ACLManager(config, app=app)

    # Créer une permission
    perm = Permission(
        resource="posts",
        action="create",
        display_name="Créer des articles",
        category="Content",
    )
    print(perm.name)  # "posts:create"

    # Vérifier une permission
    perm.matches("posts:create")  # True
    ```
"""

from typing import TYPE_CHECKING

# Type hints pour l'autocomplétion IDE (non exécuté à runtime)
if TYPE_CHECKING:
    from alak_acl.permissions.infrastructure.models.sql_model import SQLPermissionModel as SQLPermissionModel
    from alak_acl.permissions.infrastructure.models.mongo_model import MongoPermissionModel as MongoPermissionModel
    from alak_acl.permissions.infrastructure.repositories.postgresql_repository import PostgreSQLPermissionRepository as PostgreSQLPermissionRepository
    from alak_acl.permissions.infrastructure.repositories.mysql_repository import MySQLPermissionRepository as MySQLPermissionRepository
    from alak_acl.permissions.infrastructure.repositories.mongodb_repository import MongoDBPermissionRepository as MongoDBPermissionRepository

# Domain
from alak_acl.permissions.domain import (
    Permission,
    CreatePermissionDTO,
    UpdatePermissionDTO,
    PermissionResponseDTO,
    PermissionListResponseDTO,
    AssignPermissionToRoleDTO,
)

# Application - Interface
from alak_acl.permissions.application.interface.permission_repository import IPermissionRepository

# Application - Use Cases
from alak_acl.permissions.application.usecases import (
    CreatePermissionUseCase,
    UpdatePermissionUseCase,
    DeletePermissionUseCase,
    GetPermissionUseCase,
    ListPermissionsUseCase,
    SearchPermissionsUseCase,
    GetPermissionsByResourceUseCase,
    GetPermissionsByCategoryUseCase,
    CreateBulkPermissionsUseCase,
)

# Infrastructure - imports directs uniquement pour le mapper (pas de dépendance externe)
from alak_acl.permissions.infrastructure.mappers.permission_mapper import PermissionMapper

# Interface (Routes et dépendances)
from alak_acl.permissions.interface import (
    router,
    set_permission_dependencies,
    get_permission_repository,
)

# Lazy imports pour les classes qui nécessitent des dépendances externes
def __getattr__(name: str):
    """Lazy loading pour éviter les dépendances manquantes."""
    # Modèles MongoDB (nécessite motor/pymongo)
    if name == "MongoPermissionModel":
        from alak_acl.permissions.infrastructure.models.mongo_model import MongoPermissionModel
        return MongoPermissionModel
    # Modèles SQL (nécessite SQLAlchemy)
    elif name == "SQLPermissionModel":
        from alak_acl.permissions.infrastructure.models.sql_model import SQLPermissionModel
        return SQLPermissionModel
    elif name == "PostgreSQLPermissionRepository":
        from alak_acl.permissions.infrastructure.repositories.postgresql_repository import PostgreSQLPermissionRepository
        return PostgreSQLPermissionRepository
    elif name == "MySQLPermissionRepository":
        from alak_acl.permissions.infrastructure.repositories.mysql_repository import MySQLPermissionRepository
        return MySQLPermissionRepository
    elif name == "MongoDBPermissionRepository":
        from alak_acl.permissions.infrastructure.repositories.mongodb_repository import MongoDBPermissionRepository
        return MongoDBPermissionRepository
    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")


__all__ = [
    # Domain - Entité
    "Permission",
    # Domain - DTOs
    "CreatePermissionDTO",
    "UpdatePermissionDTO",
    "PermissionResponseDTO",
    "PermissionListResponseDTO",
    "AssignPermissionToRoleDTO",
    # Application - Interface
    "IPermissionRepository",
    # Application - Use Cases
    "CreatePermissionUseCase",
    "UpdatePermissionUseCase",
    "DeletePermissionUseCase",
    "GetPermissionUseCase",
    "ListPermissionsUseCase",
    "SearchPermissionsUseCase",
    "GetPermissionsByResourceUseCase",
    "GetPermissionsByCategoryUseCase",
    "CreateBulkPermissionsUseCase",
    # Infrastructure - Repositories
    "PostgreSQLPermissionRepository",
    "MySQLPermissionRepository",
    "MongoDBPermissionRepository",
    # Infrastructure - Modèles
    "SQLPermissionModel",
    "MongoPermissionModel",
    # Infrastructure - Mapper
    "PermissionMapper",
    # Interface - Router
    "router",
    # Interface - Configuration
    "set_permission_dependencies",
    "get_permission_repository",
]



"""
Use case pour la demande de réinitialisation de mot de passe.
"""

from typing import Optional

from alak_acl.auth.application.interface.auth_repository import IAuthRepository
from alak_acl.auth.application.interface.token_service import ITokenService
from alak_acl.auth.application.interface.email_service import IEmailService
from alak_acl.auth.domain.dtos.password_reset_dto import ForgotPasswordDTO
from alak_acl.shared.logging import logger


class ForgotPasswordUseCase:
    """
    Use case pour demander la réinitialisation d'un mot de passe.

    Génère un token de reset et envoie un email avec le lien.
    Pour des raisons de sécurité, retourne toujours un succès même si
    l'email n'existe pas (évite l'énumération des utilisateurs).
    """

    def __init__(
        self,
        auth_repository: IAuthRepository,
        token_service: ITokenService,
        email_service: IEmailService,
        reset_url_base: Optional[str] = None,
    ):
        """
        Initialise le use case.

        Args:
            auth_repository: Repository d'authentification
            token_service: Service de tokens JWT
            email_service: Service d'envoi d'emails
            reset_url_base: URL de base pour le lien de reset
                           (ex: https://app.example.com/reset-password)
        """
        self._auth_repository = auth_repository
        self._token_service = token_service
        self._email_service = email_service
        self._reset_url_base = reset_url_base or "http://localhost:3000/reset-password"

    async def execute(self, dto: ForgotPasswordDTO) -> bool:
        """
        Traite la demande de réinitialisation.

        Args:
            dto: DTO contenant l'email

        Returns:
            True (toujours, pour ne pas révéler si l'email existe)
        """
        logger.info(f"Demande de réinitialisation pour: {dto.email}")

        # Chercher l'utilisateur par email
        user = await self._auth_repository.get_by_email(dto.email)

        if not user:
            # Ne pas révéler que l'email n'existe pas
            logger.debug(f"Email non trouvé: {dto.email} (réponse identique)")
            return True

        if not user.is_active:
            # Compte désactivé - ne pas révéler non plus
            logger.debug(f"Compte désactivé pour: {dto.email}")
            return True

        # Générer le token de reset
        reset_token = self._token_service.create_reset_token(
            user_id=user.id,
            email=user.email,
        )

        # Construire le lien de réinitialisation
        reset_link = f"{self._reset_url_base}?token={reset_token}"

        # Envoyer l'email
        try:
            await self._email_service.send_password_reset_email(
                to=user.email,
                username=user.username,
                reset_link=reset_link,
            )
            logger.info(f"Email de réinitialisation envoyé à: {user.email}")
        except Exception as e:
            # Log l'erreur mais ne pas la propager à l'utilisateur
            logger.error(f"Erreur d'envoi email: {e}")
            # On retourne quand même True pour ne pas révéler d'info

        return True



"""
Module database - Connexions aux bases de données.

Supporte MongoDB, PostgreSQL et MySQL de manière asynchrone.

Les imports sont conditionnels pour éviter de charger des dépendances
non installées (ex: SQLAlchemy pour les utilisateurs MongoDB uniquement).
"""

from typing import TYPE_CHECKING

# Type hints pour l'autocomplétion IDE (non exécuté à runtime)
if TYPE_CHECKING:
    from alak_acl.shared.database.declarative_base import Base as Base
    from alak_acl.shared.database.postgresql import PostgreSQLDatabase as PostgreSQLDatabase
    from alak_acl.shared.database.mysql import MySQLDatabase as MySQLDatabase
    from alak_acl.shared.database.mongodb import MongoDBDatabase as MongoDBDatabase

from alak_acl.shared.database.factory import DatabaseFactory, get_database
from alak_acl.shared.database.base import BaseDatabase

# Lazy imports pour éviter les erreurs de dépendances manquantes
def __getattr__(name: str):
    """Lazy loading des classes de base de données."""
    if name == "Base":
        from alak_acl.shared.database.declarative_base import Base
        return Base
    elif name == "PostgreSQLDatabase":
        from alak_acl.shared.database.postgresql import PostgreSQLDatabase
        return PostgreSQLDatabase
    elif name == "MySQLDatabase":
        from alak_acl.shared.database.mysql import MySQLDatabase
        return MySQLDatabase
    elif name == "MongoDBDatabase":
        from alak_acl.shared.database.mongodb import MongoDBDatabase
        return MongoDBDatabase
    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")


__all__ = [
    "DatabaseFactory",
    "get_database",
    "BaseDatabase",
    # Base SQLAlchemy pour les migrations (lazy)
    "Base",
    # Classes de connexion (lazy)
    "PostgreSQLDatabase",
    "MySQLDatabase",
    "MongoDBDatabase",
]



"""
Configuration du package fastapi-acl.

Utilise Pydantic Settings pour gérer toute la configuration
via variables d'environnement ou fichier .env.
"""

from typing import Literal, Optional
from pydantic_settings import BaseSettings, SettingsConfigDict
from pydantic import Field, field_validator


class ACLConfig(BaseSettings):
    """
    Configuration principale du package ACL.

    Toutes les valeurs peuvent être définies via variables d'environnement
    avec le préfixe ACL_ (ex: ACL_DATABASE_TYPE=postgresql).

    Attributes:
        database_type: Type de base de données (mongodb, postgresql, mysql)
        mongodb_uri: URI de connexion MongoDB
        postgresql_uri: URI de connexion PostgreSQL
        mysql_uri: URI de connexion MySQL
        enable_cache: Activer le cache Redis/Memory
        redis_url: URL de connexion Redis
        cache_ttl: Durée de vie du cache en secondes
        cache_backend: Backend de cache (redis, memory)
        jwt_secret_key: Clé secrète pour signer les tokens JWT
        jwt_algorithm: Algorithme de signature JWT
        jwt_access_token_expire_minutes: Durée de vie access token
        jwt_refresh_token_expire_days: Durée de vie refresh token
        enable_api_routes: Activer l'enregistrement automatique des routes
        api_prefix: Préfixe des routes API
        enable_auth_feature: Activer la feature d'authentification
        enable_permissions_feature: Activer la feature des permissions
        enable_roles_feature: Activer la feature des rôles
        disable_auth_for_dev: Désactiver l'auth en développement
        create_default_admin: Créer un admin par défaut au démarrage
        default_admin_username: Nom d'utilisateur admin par défaut
        default_admin_email: Email admin par défaut
        default_admin_password: Mot de passe admin par défaut
        log_level: Niveau de logging
    """

    model_config = SettingsConfigDict(
        env_prefix="ACL_",
        env_file=".env",
        env_file_encoding="utf-8",
        extra="ignore",
    )

    # Database Configuration
    database_type: Literal["mongodb", "postgresql", "mysql"] = Field(
        default="postgresql",
        description="Type de base de données à utiliser"
    )
    mongodb_uri: Optional[str] = Field(
        default=None,
        description="URI de connexion MongoDB (ex: mongodb://localhost:27017/acl)"
    )
    postgresql_uri: Optional[str] = Field(
        default=None,
        description="URI de connexion PostgreSQL (ex: postgresql+asyncpg://user:pass@localhost/db)"
    )
    mysql_uri: Optional[str] = Field(
        default=None,
        description="URI de connexion MySQL (ex: mysql+asyncmy://user:pass@localhost/db)"
    )

    # Cache Configuration
    enable_cache: bool = Field(
        default=True,
        description="Activer le cache Redis/Memory"
    )
    redis_url: Optional[str] = Field(
        default=None,
        description="URL de connexion Redis (ex: redis://localhost:6379/0)"
    )
    cache_ttl: int = Field(
        default=300,
        description="Durée de vie du cache en secondes"
    )
    cache_backend: Literal["redis", "memory"] = Field(
        default="redis",
        description="Backend de cache à utiliser"
    )

    # JWT Configuration
    jwt_secret_key: str = Field(
        default="change-me-in-production-very-secret-key",
        description="Clé secrète pour signer les tokens JWT"
    )
    jwt_algorithm: str = Field(
        default="HS256",
        description="Algorithme de signature JWT"
    )
    jwt_access_token_expire_minutes: int = Field(
        default=30,
        description="Durée de vie du access token en minutes"
    )
    jwt_refresh_token_expire_days: int = Field(
        default=7,
        description="Durée de vie du refresh token en jours"
    )

    # Password Reset Configuration
    reset_token_expire_minutes: int = Field(
        default=60,
        description="Durée de vie du token de réinitialisation en minutes"
    )
    password_reset_url: Optional[str] = Field(
        default=None,
        description="URL de la page de réinitialisation du frontend (ex: https://app.com/reset-password)"
    )

    # Email/SMTP Configuration
    smtp_enabled: bool = Field(
        default=False,
        description="Activer l'envoi d'emails via SMTP (sinon utilise Console)"
    )
    smtp_server: Optional[str] = Field(
        default=None,
        description="Serveur SMTP (ex: smtp.gmail.com)"
    )
    smtp_port: int = Field(
        default=587,
        description="Port SMTP (587 pour TLS, 465 pour SSL)"
    )
    smtp_username: Optional[str] = Field(
        default=None,
        description="Nom d'utilisateur SMTP"
    )
    smtp_password: Optional[str] = Field(
        default=None,
        description="Mot de passe SMTP"
    )
    smtp_from_email: Optional[str] = Field(
        default=None,
        description="Adresse email source pour les envois"
    )
    smtp_from_name: str = Field(
        default="alak-acl",
        description="Nom affiché de l'expéditeur"
    )
    smtp_use_tls: bool = Field(
        default=True,
        description="Utiliser STARTTLS (True pour port 587)"
    )

    # API Configuration
    enable_api_routes: bool = Field(
        default=True,
        description="Activer l'enregistrement automatique des routes"
    )
    api_prefix: str = Field(
        default="/api/v1",
        description="Préfixe des routes API"
    )

    # Features Toggle
    enable_auth_feature: bool = Field(
        default=True,
        description="Activer la feature d'authentification"
    )
    enable_permissions_feature: bool = Field(
        default=False,
        description="Activer la feature des permissions"
    )
    enable_roles_feature: bool = Field(
        default=False,
        description="Activer la feature des rôles"
    )
    enable_public_registration: bool = Field(
        default=False,
        description=(
            "Activer l'API publique d'inscription (/register). "
            "True: apps classiques où les utilisateurs s'inscrivent eux-mêmes. "
            "False: SaaS multi-tenant et apps B2B où l'app gère l'inscription via ACLManager.create_account()."
        )
    )

    # Development Configuration
    disable_auth_for_dev: bool = Field(
        default=False,
        description="Désactiver l'authentification en mode développement"
    )
    create_default_admin: bool = Field(
        default=False,
        description="Créer un administrateur par défaut au démarrage"
    )
    default_admin_username: str = Field(
        default="admin",
        description="Nom d'utilisateur de l'admin par défaut"
    )
    default_admin_email: str = Field(
        default="admin@example.com",
        description="Email de l'admin par défaut"
    )
    default_admin_password: str = Field(
        default="admin123",
        description="Mot de passe de l'admin par défaut (à changer!)"
    )
    log_level: Literal["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"] = Field(
        default="INFO",
        description="Niveau de logging"
    )

    # Custom Model Configuration
    users_table_name: str = Field(
        default="acl_auth_users",
        description="Nom de la table/collection pour les utilisateurs"
    )
    extra_user_indexes: Optional[str] = Field(
        default=None,
        description="Liste des champs personnalisés à indexer (séparés par des virgules)"
    )

    def get_extra_indexes_list(self) -> list:
        """
        Retourne la liste des champs personnalisés à indexer.

        Returns:
            Liste des noms de champs
        """
        if not self.extra_user_indexes:
            return []
        return [idx.strip() for idx in self.extra_user_indexes.split(",") if idx.strip()]

    @field_validator("jwt_secret_key")
    @classmethod
    def validate_jwt_secret(cls, v: str) -> str:
        """Valide que la clé secrète JWT est suffisamment longue."""
        if len(v) < 32:
            raise ValueError("jwt_secret_key doit faire au moins 32 caractères")
        return v

    def get_database_uri(self) -> str:
        """
        Retourne l'URI de la base de données configurée.

        Returns:
            URI de connexion à la base de données

        Raises:
            ValueError: Si l'URI n'est pas configurée pour le type de DB choisi
        """
        uri_map = {
            "mongodb": self.mongodb_uri,
            "postgresql": self.postgresql_uri,
            "mysql": self.mysql_uri,
        }
        uri = uri_map.get(self.database_type)
        if not uri:
            raise ValueError(
                f"URI non configurée pour {self.database_type}. "
                f"Définissez {self.database_type}_uri."
            )
        return uri



"""
Mappers pour la feature Roles.
"""

from alak_acl.roles.infrastructure.mappers.role_mapper import RoleMapper, to_entity, to_sql_model, to_mongo_model, to_mongo_dict

__all__ = [
    "RoleMapper",
    "to_entity",
    "to_sql_model",
    "to_mongo_model",
    "to_mongo_dict",
]



"""
Modèles de base de données pour la feature Roles.

Les imports sont conditionnels (lazy) pour éviter de charger
les dépendances non installées (SQLAlchemy ou motor/pymongo).
"""

from typing import TYPE_CHECKING

# Type hints pour l'autocomplétion IDE (non exécuté à runtime)
if TYPE_CHECKING:
    from alak_acl.roles.infrastructure.models.sql_model import SQLRoleModel as SQLRoleModel
    from alak_acl.roles.infrastructure.models.sql_model import SQLUserRoleModel as SQLUserRoleModel
    from alak_acl.roles.infrastructure.models.sql_model import SQLMembershipModel as SQLMembershipModel
    from alak_acl.roles.infrastructure.models.mongo_model import MongoRoleModel as MongoRoleModel
    from alak_acl.roles.infrastructure.models.mongo_model import MongoUserRoleModel as MongoUserRoleModel


# Lazy imports pour éviter les erreurs de dépendances manquantes
def __getattr__(name: str):
    """Lazy loading des classes pour éviter les dépendances manquantes."""
    # Modèles MongoDB (nécessite motor/pymongo)
    if name == "MongoRoleModel":
        from alak_acl.roles.infrastructure.models.mongo_model import MongoRoleModel
        return MongoRoleModel
    elif name == "MongoUserRoleModel":
        from alak_acl.roles.infrastructure.models.mongo_model import MongoUserRoleModel
        return MongoUserRoleModel
    # Modèles SQL (nécessite SQLAlchemy)
    elif name == "SQLRoleModel":
        from alak_acl.roles.infrastructure.models.sql_model import SQLRoleModel
        return SQLRoleModel
    elif name == "SQLUserRoleModel":
        from alak_acl.roles.infrastructure.models.sql_model import SQLUserRoleModel
        return SQLUserRoleModel
    elif name == "SQLMembershipModel":
        from alak_acl.roles.infrastructure.models.sql_model import SQLMembershipModel
        return SQLMembershipModel
    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")


__all__ = [
    # Modèles SQL
    "SQLRoleModel",
    "SQLUserRoleModel",
    "SQLMembershipModel",
    # Modèles MongoDB
    "MongoRoleModel",
    "MongoUserRoleModel",
]



"""
Use cases pour la gestion des rôles.
"""

from datetime import datetime
from typing import List, Optional

from alak_acl.roles.application.interface.role_repository import IRoleRepository
from alak_acl.roles.domain.dtos.role_dto import CreateRoleDTO, UpdateRoleDTO
from alak_acl.roles.domain.entities.role import Role
from alak_acl.shared.exceptions import RoleAlreadyExistsError, RoleNotFoundError



class CreateRoleUseCase:
    """Use case pour créer un nouveau rôle."""

    def __init__(self, role_repository: IRoleRepository):
        self._role_repository = role_repository

    async def execute(self, dto: CreateRoleDTO) -> Role:
        """
        Crée un nouveau rôle.

        Args:
            dto: Données du rôle à créer

        Returns:
            Rôle créé

        Raises:
            RoleAlreadyExistsError: Si un rôle avec ce nom existe déjà
        """
        # Vérifier si le nom existe déjà
        if await self._role_repository.role_exists(dto.name):
            raise RoleAlreadyExistsError("name",f"Un rôle avec le nom '{dto.name}' existe déjà")

        role = Role(
            name=dto.name,
            display_name=dto.display_name,
            description=dto.description,
            permissions=dto.permissions,
            is_default=dto.is_default,
            priority=dto.priority,
            tenant_id=dto.tenant_id,
        )

        return await self._role_repository.create_role(role)


class UpdateRoleUseCase:
    """Use case pour mettre à jour un rôle."""

    def __init__(self, role_repository: IRoleRepository):
        self._role_repository = role_repository

    async def execute(self, role_id: str, dto: UpdateRoleDTO) -> Role:
        """
        Met à jour un rôle.

        Args:
            role_id: ID du rôle
            dto: Données à mettre à jour

        Returns:
            Rôle mis à jour

        Raises:
            RoleNotFoundError: Si le rôle n'existe pas
        """
        role = await self._role_repository.get_by_id(role_id)
        if not role:
            raise RoleNotFoundError("id", f"Rôle non trouvé: {role_id}")

        # Mettre à jour les champs fournis
        if dto.display_name is not None:
            role.display_name = dto.display_name
        if dto.description is not None:
            role.description = dto.description
        if dto.permissions is not None:
            role.permissions = dto.permissions
        if dto.is_active is not None:
            role.is_active = dto.is_active
        if dto.is_default is not None:
            role.is_default = dto.is_default
        if dto.priority is not None:
            role.priority = dto.priority
        if dto.tenant_id is not None:
            role.tenant_id = dto.tenant_id

        role.updated_at = datetime.utcnow()

        return await self._role_repository.update_role(role)


class DeleteRoleUseCase:
    """Use case pour supprimer un rôle."""

    def __init__(self, role_repository: IRoleRepository):
        self._role_repository = role_repository

    async def execute(self, role_id: str) -> bool:
        """
        Supprime un rôle.

        Args:
            role_id: ID du rôle

        Returns:
            True si supprimé

        Raises:
            RoleNotFoundError: Si le rôle n'existe pas
            PermissionDeniedError: Si c'est un rôle système
        """
        return await self._role_repository.delete_role(role_id)


class GetRoleUseCase:
    """Use case pour récupérer un rôle."""

    def __init__(self, role_repository: IRoleRepository):
        self._role_repository = role_repository

    async def execute_by_id(self, role_id: str) -> Optional[Role]:
        """Récupère un rôle par son ID."""
        return await self._role_repository.get_by_id(role_id)

    async def execute_by_name(self, name: str) -> Optional[Role]:
        """Récupère un rôle par son nom."""
        return await self._role_repository.get_by_name(name)


class ListRolesUseCase:
    """Use case pour lister les rôles."""

    def __init__(self, role_repository: IRoleRepository):
        self._role_repository = role_repository

    async def execute(
        self,
        skip: int = 0,
        limit: int = 100,
        is_active: Optional[bool] = None,
        tenant_id: Optional[str] = None,
    ) -> tuple[List[Role], int]:
        """
        Liste les rôles avec pagination.

        Args:
            skip: Nombre d'éléments à sauter
            limit: Limite de résultats
            is_active: Filtrer par statut actif
            tenant_id: Filtrer par tenant (mode multi-tenant)

        Returns:
            Tuple (liste des rôles, total)
        """
        roles = await self._role_repository.list_roles(
            skip=skip,
            limit=limit,
            is_active=is_active,
            tenant_id=tenant_id,
        )
        total = await self._role_repository.count_roles(
            is_active=is_active,
            tenant_id=tenant_id,
        )
        return roles, total


class AssignRoleUseCase:
    """Use case pour assigner un rôle à un utilisateur."""

    def __init__(self, role_repository: IRoleRepository):
        self._role_repository = role_repository

    async def execute(self, user_id: str, role_id: str) -> bool:
        """
        Assigne un rôle à un utilisateur.

        Args:
            user_id: ID de l'utilisateur
            role_id: ID du rôle

        Returns:
            True si assigné

        Raises:
            RoleNotFoundError: Si le rôle n'existe pas
        """
        return await self._role_repository.assign_role_to_user(user_id, role_id)

    async def execute_by_name(self, user_id: str, role_name: str) -> bool:
        """
        Assigne un rôle par son nom.

        Args:
            user_id: ID de l'utilisateur
            role_name: Nom du rôle

        Returns:
            True si assigné

        Raises:
            RoleNotFoundError: Si le rôle n'existe pas
        """
        role = await self._role_repository.get_by_name(role_name)
        if not role:
            raise RoleNotFoundError("name", f"Rôle non trouvé: {role_name}")
        return await self._role_repository.assign_role_to_user(user_id, role.id)


class RemoveRoleUseCase:
    """Use case pour retirer un rôle d'un utilisateur."""

    def __init__(self, role_repository: IRoleRepository):
        self._role_repository = role_repository

    async def execute(self, user_id: str, role_id: str) -> bool:
        """
        Retire un rôle d'un utilisateur.

        Args:
            user_id: ID de l'utilisateur
            role_id: ID du rôle

        Returns:
            True si retiré
        """
        return await self._role_repository.remove_role_from_user(user_id, role_id)


class GetUserRolesUseCase:
    """Use case pour récupérer les rôles d'un utilisateur."""

    def __init__(self, role_repository: IRoleRepository):
        self._role_repository = role_repository

    async def execute(self, user_id: str) -> List[Role]:
        """
        Récupère les rôles d'un utilisateur.

        Args:
            user_id: ID de l'utilisateur

        Returns:
            Liste des rôles
        """
        return await self._role_repository.get_user_roles(user_id)


class GetUserPermissionsUseCase:
    """Use case pour récupérer les permissions d'un utilisateur."""

    def __init__(self, role_repository: IRoleRepository):
        self._role_repository = role_repository

    async def execute(self, user_id: str) -> List[str]:
        """
        Récupère toutes les permissions d'un utilisateur.

        Args:
            user_id: ID de l'utilisateur

        Returns:
            Liste des permissions uniques
        """
        return await self._role_repository.get_user_permissions(user_id)


class CheckPermissionUseCase:
    """Use case pour vérifier si un utilisateur a une permission."""

    def __init__(self, role_repository: IRoleRepository):
        self._role_repository = role_repository

    async def execute(self, user_id: str, permission: str) -> bool:
        """
        Vérifie si un utilisateur a une permission.

        Args:
            user_id: ID de l'utilisateur
            permission: Permission à vérifier

        Returns:
            True si l'utilisateur a la permission
        """
        roles = await self._role_repository.get_user_roles(user_id)
        for role in roles:
            if role.is_active and role.has_permission(permission):
                return True
        return False


class CheckRoleUseCase:
    """Use case pour vérifier si un utilisateur a un rôle."""

    def __init__(self, role_repository: IRoleRepository):
        self._role_repository = role_repository

    async def execute(self, user_id: str, role_id: str) -> bool:
        """Vérifie si un utilisateur a un rôle par ID."""
        return await self._role_repository.user_has_role(user_id, role_id)

    async def execute_by_name(self, user_id: str, role_name: str) -> bool:
        """Vérifie si un utilisateur a un rôle par nom."""
        return await self._role_repository.user_has_role_by_name(user_id, role_name)


class SetUserRolesUseCase:
    """Use case pour définir la liste complète des rôles d'un utilisateur."""

    def __init__(self, role_repository: IRoleRepository):
        self._role_repository = role_repository

    async def execute(self, user_id: str, role_ids: List[str]) -> bool:
        """
        Définit tous les rôles d'un utilisateur.

        Args:
            user_id: ID de l'utilisateur
            role_ids: Liste des IDs de rôles

        Returns:
            True si succès
        """
        return await self._role_repository.set_user_roles(user_id, role_ids)


class AssignDefaultRolesUseCase:
    """Use case pour assigner les rôles par défaut à un nouvel utilisateur."""

    def __init__(self, role_repository: IRoleRepository):
        self._role_repository = role_repository

    async def execute(self, user_id: str) -> List[Role]:
        """
        Assigne les rôles par défaut à un utilisateur.

        Args:
            user_id: ID de l'utilisateur

        Returns:
            Liste des rôles assignés
        """
        default_roles = await self._role_repository.get_default_roles()

        for role in default_roles:
            await self._role_repository.assign_role_to_user(user_id, role.id)

        return default_roles



"""
Modèles SQLAlchemy pour les rôles et memberships (PostgreSQL/MySQL).

Le modèle Membership représente l'appartenance d'un utilisateur à un tenant
avec un rôle spécifique. C'est la table pivot du système SaaS multi-tenant.
"""

from datetime import datetime
from uuid import uuid4

from sqlalchemy import Column, String, Boolean, DateTime, JSON, Integer, ForeignKey, UniqueConstraint
from sqlalchemy.orm import relationship

from alak_acl.shared.database.declarative_base import Base


def generate_uuid_str() -> str:
    """Génère un UUID sous forme de string."""
    return str(uuid4())


class SQLRoleModel(Base):
    """
    Modèle SQLAlchemy pour la table des rôles.

    Compatible PostgreSQL et MySQL.

    Les rôles peuvent être:
    - Globaux (tenant_id=None): disponibles pour tous les tenants
    - Spécifiques à un tenant (tenant_id set): créés par l'app hôte pour un tenant

    Attributes:
        id: Identifiant unique (VARCHAR(36))
        name: Nom du rôle (unique par tenant ou globalement si tenant_id=None)
        display_name: Nom d'affichage
        description: Description du rôle
        permissions: Liste des permissions (JSON)
        is_active: Rôle actif
        is_default: Rôle par défaut pour les nouveaux membres
        is_system: Rôle système (non supprimable)
        priority: Priorité du rôle
        tenant_id: Identifiant du tenant (None = rôle global)
        created_at: Date de création
        updated_at: Date de mise à jour
    """

    __tablename__ = "acl_roles"
    __table_args__ = (
        # Index unique composite : un nom de rôle est unique par tenant
        # Si tenant_id=None, le nom est unique parmi les rôles globaux
        UniqueConstraint('tenant_id', 'name', name='uq_role_tenant_name'),
        {'extend_existing': True},
    )

    id = Column(
        String(36),
        primary_key=True,
        default=generate_uuid_str,
        index=True,
    )
    name = Column(
        String(50),
        nullable=False,
        index=True,
    )
    display_name = Column(
        String(100),
        nullable=True,
    )
    description = Column(
        String(500),
        nullable=True,
    )
    permissions = Column(
        JSON,
        nullable=False,
        default=list,
    )
    is_active = Column(
        Boolean,
        default=True,
        nullable=False,
        index=True,
    )
    is_default = Column(
        Boolean,
        default=False,
        nullable=False,
        index=True,
    )
    is_system = Column(
        Boolean,
        default=False,
        nullable=False,
    )
    priority = Column(
        Integer,
        default=0,
        nullable=False,
    )
    tenant_id = Column(
        String(36),
        nullable=True,
        index=True,
    )
    created_at = Column(
        DateTime,
        default=datetime.utcnow,
        nullable=False,
    )
    updated_at = Column(
        DateTime,
        default=datetime.utcnow,
        onupdate=datetime.utcnow,
        nullable=False,
    )

    # Relationship avec les memberships
    memberships = relationship(
        "SQLMembershipModel",
        back_populates="role",
        lazy="selectin",
    )

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__}(id={self.id}, name={self.name})>"


class SQLMembershipModel(Base):
    """
    Modèle pour la table de membership (pivot user-tenant-role).

    Représente l'appartenance d'un utilisateur à un tenant avec un rôle.
    C'est le cœur du système SaaS multi-tenant.

    Un utilisateur peut:
    - Appartenir à plusieurs tenants
    - Avoir plusieurs rôles dans un même tenant
    - Avoir des rôles différents selon le tenant
    - Avoir des rôles globaux (tenant_id=NULL) assignés lors de l'inscription

    Attributes:
        user_id: FK vers l'utilisateur
        tenant_id: ID du tenant (NULL pour rôle global, sinon fourni par l'app hôte)
        role_id: FK vers le rôle
        assigned_at: Date d'assignation
        assigned_by: ID de l'utilisateur ayant fait l'assignation (optionnel)

    Note:
        tenant_id n'a pas de FK car les tenants sont gérés par l'app hôte,
        pas par le package ACL. Il est nullable pour permettre les rôles globaux.
    """

    __tablename__ = "acl_memberships"
    __table_args__ = (
        # Un utilisateur ne peut avoir le même rôle qu'une fois par tenant
        UniqueConstraint('user_id', 'tenant_id', 'role_id', name='uq_membership'),
        {'extend_existing': True},
    )

    id = Column(
        String(36),
        primary_key=True,
        default=generate_uuid_str,
        index=True,
    )
    user_id = Column(
        String(36),
        ForeignKey('acl_auth_users.id', ondelete='CASCADE'),
        nullable=False,
        index=True,
    )
    tenant_id = Column(
        String(36),
        nullable=True,  # Nullable pour les rôles globaux (sans tenant)
        index=True,
    )
    role_id = Column(
        String(36),
        ForeignKey('acl_roles.id', ondelete='CASCADE'),
        nullable=False,
        index=True,
    )
    assigned_at = Column(
        DateTime,
        default=datetime.utcnow,
        nullable=False,
    )
    assigned_by = Column(
        String(36),
        nullable=True,  # Optionnel: qui a assigné ce rôle
    )

    # Relationships
    user = relationship(
        "SQLAuthUserModel",
        back_populates="memberships",
    )
    role = relationship(
        "SQLRoleModel",
        back_populates="memberships",
    )

    def __repr__(self) -> str:
        return f"<Membership(user={self.user_id}, tenant={self.tenant_id}, role={self.role_id})>"


# Alias pour rétrocompatibilité (déprécié)
SQLUserRoleModel = SQLMembershipModel



"""
ACLManager - Point d'entrée principal du package fastapi-acl.

Ce module fournit le gestionnaire principal qui initialise
toutes les connexions et enregistre automatiquement les routes.

Supporte les modèles utilisateur personnalisés pour ajouter
des champs/colonnes supplémentaires.
"""

from typing import Optional, Union, Type, List, TYPE_CHECKING

from fastapi import FastAPI

from alak_acl.shared.config import ACLConfig
from alak_acl.shared.database.factory import DatabaseFactory, set_database, DatabaseType
from alak_acl.shared.cache.factory import CacheFactory, set_cache, CacheType
from alak_acl.shared.exceptions import ConfigurationError
from alak_acl.shared.logging import logger, get_logger

from alak_acl.auth.application.interface.auth_repository import IAuthRepository
from alak_acl.auth.application.interface.token_service import ITokenService
from alak_acl.auth.application.interface.password_hasher import IPasswordHasher
from alak_acl.auth.infrastructure.services.jwt_token_service import JWTTokenService
from alak_acl.auth.infrastructure.services.argon2_password_hasher import Argon2PasswordHasher
from alak_acl.auth.infrastructure.mappers.auth_user_mapper import AuthUserMapper
from alak_acl.auth.interface.dependencies import set_auth_dependencies, set_email_dependencies
from alak_acl.auth.application.interface.email_service import IEmailService
from alak_acl.auth.interface.api import router as auth_router
from alak_acl.auth.interface.admin.routes import router as admin_router
from alak_acl.auth.domain.dtos.register_dto import RegisterDTO
from alak_acl.auth.application.usecases.register_usecase import RegisterUseCase

# Roles imports (interfaces uniquement - pas de dépendance DB)
from alak_acl.roles.application.interface.role_repository import IRoleRepository
from alak_acl.roles.interface.dependencies import set_role_dependencies
from alak_acl.roles.interface.routes import router as roles_router
from alak_acl.roles.domain.entities.role import Role

# Permissions imports (interfaces uniquement - pas de dépendance DB)
from alak_acl.permissions.application.interface.permission_repository import IPermissionRepository
from alak_acl.permissions.interface.dependencies import set_permission_dependencies
from alak_acl.permissions.interface.routes import router as permissions_router
from alak_acl.permissions.domain.entities.permission import Permission

# Type hints uniquement (pas chargés au runtime)
if TYPE_CHECKING:
    from alak_acl.shared.database.mongodb import MongoDBDatabase
    from alak_acl.shared.database.postgresql import PostgreSQLDatabase
    from alak_acl.shared.database.mysql import MySQLDatabase
    from alak_acl.auth.infrastructure.models.sql_model import SQLAuthUserModel
    from alak_acl.auth.infrastructure.models.mongo_model import MongoAuthUserModel


class ACLManager:
    """
    Gestionnaire principal du package fastapi-acl.

    Point d'entrée unique pour initialiser et configurer
    toute l'infrastructure ACL.

    Attributes:
        config: Configuration ACL
        app: Instance FastAPI (optionnelle)
        database: Connexion base de données
        cache: Backend de cache
        auth_repository: Repository d'authentification
        token_service: Service de tokens JWT
        password_hasher: Service de hashage

    Example:
        ```python
        from fastapi import FastAPI
        from alak_acl import ACLManager, ACLConfig

        app = FastAPI()
        config = ACLConfig(
            database_type="postgresql",
            postgresql_uri="postgresql+asyncpg://user:pass@localhost/db",
        )

        acl = ACLManager(config, app=app)

        @app.on_event("startup")
        async def startup():
            await acl.initialize()

        @app.on_event("shutdown")
        async def shutdown():
            await acl.close()
        ```

    Example avec modèle personnalisé:
        ```python
        from sqlalchemy import Column, String, Integer
        from alak_acl import ACLManager, ACLConfig
        from alak_acl.auth.infrastructure.models import SQLAuthUserModel

        # Définir un modèle personnalisé avec des colonnes supplémentaires
        class CustomUserModel(SQLAuthUserModel):
            __tablename__ = "users"
            phone = Column(String(20), nullable=True)
            company = Column(String(100), nullable=True)

        acl = ACLManager(
            config=config,
            app=app,
            sql_user_model=CustomUserModel,
        )
        ```
    """

    def __init__(
        self,
        config: ACLConfig,
        app: Optional[FastAPI] = None,
        sql_user_model: Optional[Type["SQLAuthUserModel"]] = None,
        mongo_user_model: Optional[Type["MongoAuthUserModel"]] = None,
        extra_user_indexes: Optional[List[str]] = None,
    ):
        """
        Initialise le gestionnaire ACL.

        Args:
            config: Configuration ACL
            app: Instance FastAPI pour l'auto-registration des routes
            sql_user_model: Classe de modèle SQL personnalisée (pour PostgreSQL/MySQL)
            mongo_user_model: Classe de modèle MongoDB personnalisée
            extra_user_indexes: Liste des champs personnalisés à indexer (MongoDB)
        """
        self._config = config
        self._app = app
        self._initialized = False

        # Modèles personnalisés (chargés uniquement si nécessaire selon le type de DB)
        self._sql_user_model = sql_user_model
        if self._sql_user_model is None and config.database_type in ("postgresql", "mysql"):
            from alak_acl.auth.infrastructure.models.sql_model import SQLAuthUserModel
            self._sql_user_model = SQLAuthUserModel

        # Modèle Mongo (chargé uniquement si MongoDB est utilisé)
        self._mongo_user_model = mongo_user_model
        if self._mongo_user_model is None and config.database_type == "mongodb":
            from alak_acl.auth.infrastructure.models.mongo_model import MongoAuthUserModel
            self._mongo_user_model = MongoAuthUserModel
        self._extra_user_indexes = extra_user_indexes or self._parse_extra_indexes()

        # Infrastructure (initialisées dans initialize())
        self._database: Optional[DatabaseType] = None
        self._cache: Optional[CacheType] = None

        # Services Auth (initialisés dans initialize())
        self._auth_repository: Optional[IAuthRepository] = None
        self._token_service: Optional[ITokenService] = None
        self._password_hasher: Optional[IPasswordHasher] = None

        # Services Roles (initialisés dans initialize())
        self._role_repository: Optional[IRoleRepository] = None

        # Services Permissions (initialisés dans initialize())
        self._permission_repository: Optional[IPermissionRepository] = None

        # Service Email (initialisé dans initialize())
        self._email_service: Optional[IEmailService] = None

        # Configurer le logger
        get_logger(level=config.log_level)

        # Auto-register des routes si app fournie
        if app and config.enable_api_routes:
            self._register_routes(app)
            self._register_exception_handlers(app)

            
            
    def _register_exception_handlers(self, app: FastAPI):
        from alak_acl.shared.exception_handlers import register_exception_handlers
        register_exception_handlers(app)

    def _parse_extra_indexes(self) -> List[str]:
        """Parse les index supplémentaires depuis la config."""
        if self._config.extra_user_indexes:
            return [
                idx.strip()
                for idx in self._config.extra_user_indexes.split(",")
                if idx.strip()
            ]
        return []

    def _register_routes(self, app: FastAPI) -> None:
        """
        Enregistre automatiquement les routes dans FastAPI.

        Args:
            app: Instance FastAPI
        """
        prefix = self._config.api_prefix

        if self._config.enable_auth_feature:
            app.include_router(auth_router, prefix=prefix)
            app.include_router(admin_router, prefix=prefix)
            logger.info(f"Routes auth enregistrées avec préfixe: {prefix}")

        # Feature roles
        if self._config.enable_roles_feature:
            app.include_router(roles_router, prefix=prefix)
            logger.info(f"Routes roles enregistrées avec préfixe: {prefix}")

        # Feature permissions
        if self._config.enable_permissions_feature:
            app.include_router(permissions_router, prefix=prefix)
            logger.info(f"Routes permissions enregistrées avec préfixe: {prefix}")

    async def initialize(self) -> None:
        """
        Initialise toutes les connexions et services.

        Cette méthode doit être appelée au démarrage de l'application,
        typiquement dans l'événement startup de FastAPI.

        Raises:
            ConfigurationError: Si l'initialisation échoue
        """
        if self._initialized:
            logger.warning("ACLManager déjà initialisé")
            return

        logger.info("Initialisation de ACLManager...")

        try:
            # 1. Initialiser la base de données
            await self._init_database()

            # 2. Initialiser le cache
            await self._init_cache()

            # 3. Initialiser les services Auth
            if self._config.enable_auth_feature:
                await self._init_auth_services()

            # 3b. Initialiser le service Email
            if self._config.enable_auth_feature:
                self._init_email_service()

            # 4. Initialiser les services Roles
            if self._config.enable_roles_feature:
                await self._init_role_services()

            # 5. Créer l'admin par défaut si configuré
            if self._config.create_default_admin:
                await self._create_default_admin()

            # 6. Créer les rôles par défaut si configurés
            if self._config.enable_roles_feature:
                await self._create_default_roles()

            # 7. Initialiser les services Permissions
            if self._config.enable_permissions_feature:
                await self._init_permission_services()

            # 8. Créer les permissions par défaut
            if self._config.enable_permissions_feature:
                await self._create_default_permissions()

            self._initialized = True
            logger.info("ACLManager initialisé avec succès")

        except Exception as e:
            logger.error(f"Erreur d'initialisation: {e}")
            await self.close()
            raise ConfigurationError(f"Échec de l'initialisation: {e}")

    async def _init_database(self) -> None:
        """Initialise la connexion à la base de données."""
        self._database = DatabaseFactory.create(self._config)
        await self._database.connect()
        set_database(self._database)

        # Créer les tables si PostgreSQL/MySQL
        if self._config.database_type in ("postgresql", "mysql"):
            # Importer tous les modèles SQL pour qu'ils soient enregistrés dans le metadata
            # AVANT de créer les tables (sinon seules certaines tables seraient créées)
            from alak_acl.auth.infrastructure.models.sql_model import SQLAuthUserModel  # noqa: F401
            from alak_acl.roles.infrastructure.models.sql_model import SQLRoleModel, SQLMembershipModel  # noqa: F401
            from alak_acl.permissions.infrastructure.models.sql_model import SQLPermissionModel  # noqa: F401

            await self._database.create_tables()

    async def _init_cache(self) -> None:
        """Initialise le cache."""
        self._cache = await CacheFactory.create(self._config)
        set_cache(self._cache)

    async def _init_auth_services(self) -> None:
        """
        Initialise les services d'authentification.

        Utilise les modèles personnalisés si fournis.
        """
        # Password hasher
        self._password_hasher = Argon2PasswordHasher()

        # Token service
        self._token_service = JWTTokenService(self._config)

        # Créer le mapper avec les modèles personnalisés
        mapper = AuthUserMapper(
            sql_model_class=self._sql_user_model,
            mongo_model_class=self._mongo_user_model,
        )

        # Repository selon le type de DB avec modèle personnalisé
        db_type = self._config.database_type

        if db_type == "mongodb":
            from alak_acl.auth.infrastructure.repositories.mongodb_repository import MongoDBAuthRepository
            collection_name = self._config.users_table_name or "auth_users"
            self._auth_repository = MongoDBAuthRepository(
                db=self._database,
                collection_name=collection_name,
                model_class=self._mongo_user_model,
                mapper=mapper,
            )
            # Créer les index MongoDB (avec les index personnalisés)
            await self._auth_repository.create_indexes(
                extra_indexes=self._extra_user_indexes
            )
            logger.debug(f"MongoDB collection: {collection_name}")

        elif db_type == "postgresql":
            from alak_acl.auth.infrastructure.repositories.postgresql_repository import PostgreSQLAuthRepository
            self._auth_repository = PostgreSQLAuthRepository(
                db=self._database,
                model_class=self._sql_user_model,
                mapper=mapper,
            )
            logger.debug(f"PostgreSQL model: {self._sql_user_model.__name__}")

        elif db_type == "mysql":
            from alak_acl.auth.infrastructure.repositories.mysql_repository import MySQLAuthRepository
            self._auth_repository = MySQLAuthRepository(
                db=self._database,
                model_class=self._sql_user_model,
                mapper=mapper,
            )
            logger.debug(f"MySQL model: {self._sql_user_model.__name__}")

        # Configurer les dépendances FastAPI (role_repository sera ajouté après son initialisation)
        set_auth_dependencies(
            auth_repository=self._auth_repository,
            token_service=self._token_service,
            password_hasher=self._password_hasher,
            role_repository=None,  # Sera mis à jour après _init_role_services
            config=self._config,
        )

        logger.info("Services d'authentification initialisés")

    def _init_email_service(self) -> None:
        """
        Initialise le service d'email pour le reset de mot de passe.

        Utilise SMTP si configuré, sinon ConsoleEmailService pour le dev.
        """
        if self._config.smtp_enabled and self._config.smtp_server:
            # Service SMTP pour la production
            from alak_acl.auth.infrastructure.services.smtp_email_service import SMTPEmailService
            self._email_service = SMTPEmailService(
                smtp_server=self._config.smtp_server,
                smtp_port=self._config.smtp_port,
                smtp_username=self._config.smtp_username,
                smtp_password=self._config.smtp_password,
                from_email=self._config.smtp_from_email or "noreply@example.com",
                from_name=self._config.smtp_from_name,
                use_tls=self._config.smtp_use_tls,
            )
            logger.info("Service email SMTP initialisé")
        else:
            # Service Console pour le développement
            from alak_acl.auth.infrastructure.services.console_email_service import ConsoleEmailService
            self._email_service = ConsoleEmailService(
                app_name=self._config.smtp_from_name,
            )
            logger.info("Service email Console initialisé (mode développement)")

        # Configurer les dépendances email
        set_email_dependencies(
            email_service=self._email_service,
            reset_url_base=self._config.password_reset_url,
        )

    async def _init_role_services(self) -> None:
        """
        Initialise les services de gestion des rôles.
        """
        # Repository selon le type de DB
        db_type = self._config.database_type

        if db_type == "mongodb":
            from alak_acl.roles.infrastructure.repositories.mongodb_repository import MongoDBRoleRepository
            self._role_repository = MongoDBRoleRepository(
                db=self._database,
                roles_collection="acl_roles",
                user_roles_collection="acl_user_roles",
            )
            # Créer les index MongoDB
            await self._role_repository.create_indexes()
            logger.debug("MongoDB role repository initialisé")

        elif db_type == "postgresql":
            from alak_acl.roles.infrastructure.repositories.postgresql_repository import PostgreSQLRoleRepository
            self._role_repository = PostgreSQLRoleRepository(
                db=self._database,
            )
            logger.debug("PostgreSQL role repository initialisé")

        elif db_type == "mysql":
            from alak_acl.roles.infrastructure.repositories.mysql_repository import MySQLRoleRepository
            self._role_repository = MySQLRoleRepository(
                db=self._database,
            )
            logger.debug("MySQL role repository initialisé")

        # Configurer les dépendances FastAPI pour les rôles
        set_role_dependencies(
            role_repository=self._role_repository,
        )

        # Mettre à jour les dépendances auth avec le role_repository
        if self._auth_repository and self._token_service and self._password_hasher:
            set_auth_dependencies(
                auth_repository=self._auth_repository,
                token_service=self._token_service,
                password_hasher=self._password_hasher,
                role_repository=self._role_repository,
                config=self._config,
            )

        logger.info("Services de rôles initialisés")

    async def _init_permission_services(self) -> None:
        """
        Initialise les services de gestion des permissions.
        """
        # Repository selon le type de DB
        db_type = self._config.database_type

        if db_type == "mongodb":
            from alak_acl.permissions.infrastructure.repositories.mongodb_repository import MongoDBPermissionRepository
            self._permission_repository = MongoDBPermissionRepository(
                db=self._database,
                collection_name="acl_permissions",
            )
            # Créer les index MongoDB
            await self._permission_repository.create_indexes()
            logger.debug("MongoDB permission repository initialisé")

        elif db_type == "postgresql":
            from alak_acl.permissions.infrastructure.repositories.postgresql_repository import PostgreSQLPermissionRepository
            self._permission_repository = PostgreSQLPermissionRepository(
                db=self._database,
            )
            logger.debug("PostgreSQL permission repository initialisé")

        elif db_type == "mysql":
            from alak_acl.permissions.infrastructure.repositories.mysql_repository import MySQLPermissionRepository
            self._permission_repository = MySQLPermissionRepository(
                db=self._database,
            )
            logger.debug("MySQL permission repository initialisé")

        # Configurer les dépendances FastAPI
        set_permission_dependencies(
            permission_repository=self._permission_repository,
        )

        logger.info("Services de permissions initialisés")

    async def _create_default_permissions(self) -> None:
        """
        Crée les permissions par défaut si elles n'existent pas.
        """
        if not self._permission_repository:
            return

        default_permissions = [
            # Permissions profile
            Permission(
                resource="profile",
                action="read",
                display_name="Voir son profil",
                description="Permet de consulter son propre profil",
                category="User",
                is_system=True,
            ),
            Permission(
                resource="profile",
                action="update",
                display_name="Modifier son profil",
                description="Permet de modifier son propre profil",
                category="User",
                is_system=True,
            ),
            # Permissions users (admin)
            Permission(
                resource="users",
                action="read",
                display_name="Voir les utilisateurs",
                description="Permet de consulter la liste des utilisateurs",
                category="Admin",
                is_system=True,
            ),
            Permission(
                resource="users",
                action="create",
                display_name="Créer des utilisateurs",
                description="Permet de créer de nouveaux utilisateurs",
                category="Admin",
                is_system=True,
            ),
            Permission(
                resource="users",
                action="update",
                display_name="Modifier les utilisateurs",
                description="Permet de modifier les utilisateurs",
                category="Admin",
                is_system=True,
            ),
            Permission(
                resource="users",
                action="delete",
                display_name="Supprimer des utilisateurs",
                description="Permet de supprimer des utilisateurs",
                category="Admin",
                is_system=True,
            ),
            # Permissions roles (admin)
            Permission(
                resource="roles",
                action="read",
                display_name="Voir les rôles",
                description="Permet de consulter les rôles",
                category="Admin",
                is_system=True,
            ),
            Permission(
                resource="roles",
                action="create",
                display_name="Créer des rôles",
                description="Permet de créer de nouveaux rôles",
                category="Admin",
                is_system=True,
            ),
            Permission(
                resource="roles",
                action="update",
                display_name="Modifier les rôles",
                description="Permet de modifier les rôles",
                category="Admin",
                is_system=True,
            ),
            Permission(
                resource="roles",
                action="delete",
                display_name="Supprimer des rôles",
                description="Permet de supprimer des rôles",
                category="Admin",
                is_system=True,
            ),
            Permission(
                resource="roles",
                action="assign",
                display_name="Assigner des rôles",
                description="Permet d'assigner des rôles aux utilisateurs",
                category="Admin",
                is_system=True,
            ),
            # Permissions système
            Permission(
                resource="permissions",
                action="read",
                display_name="Voir les permissions",
                description="Permet de consulter les permissions",
                category="Admin",
                is_system=True,
            ),
            Permission(
                resource="permissions",
                action="manage",
                display_name="Gérer les permissions",
                description="Permet de créer/modifier/supprimer les permissions",
                category="Admin",
                is_system=True,
            ),
        ]

        created_count = 0
        for permission in default_permissions:
            exists = await self._permission_repository.permission_exists(permission.name)
            if not exists:
                try:
                    await self._permission_repository.create_permission(permission)
                    created_count += 1
                except Exception as e:
                    logger.warning(
                        f"Impossible de créer la permission {permission.name}: {e}"
                    )

        if created_count > 0:
            logger.info(f"{created_count} permissions par défaut créées")

    async def _create_default_roles(self) -> None:
        """
        Crée les rôles par défaut (admin, user) si ils n'existent pas.
        """
        if not self._role_repository:
            return

        default_roles = [
            Role(
                name="admin",
                display_name="Administrateur",
                description="Rôle administrateur avec tous les droits",
                permissions=["*"],
                is_system=True,
                is_default=False,
                priority=100,
            ),
            Role(
                name="user",
                display_name="Utilisateur",
                description="Rôle utilisateur standard",
                permissions=["profile:read", "profile:update"],
                is_system=True,
                is_default=True,
                priority=1,
            ),
        ]

        for role in default_roles:
            existing = await self._role_repository.get_by_name(role.name)
            if not existing:
                try:
                    await self._role_repository.create_role(role)
                    logger.info(f"Rôle par défaut créé: {role.name}")
                except Exception as e:
                    logger.warning(f"Impossible de créer le rôle {role.name}: {e}")
            else:
                logger.debug(f"Rôle {role.name} existe déjà")

    async def _create_default_admin(self) -> None:
        """
        Crée l'administrateur par défaut si configuré.

        Note: L'admin par défaut est créé comme superuser (is_superuser=True),
        ce qui lui donne tous les droits indépendamment des rôles/tenants.
        Il n'est pas assigné à un tenant car c'est un compte système.
        """
        if not self._auth_repository or not self._password_hasher:
            return

        # Vérifier si l'admin existe déjà
        existing = await self._auth_repository.get_by_username(
            self._config.default_admin_username
        )
        if existing:
            logger.debug("Admin par défaut existe déjà")
            return

        try:
            register_dto = RegisterDTO(
                username=self._config.default_admin_username,
                email=self._config.default_admin_email,
                password=self._config.default_admin_password,
            )

            register_usecase = RegisterUseCase(
                auth_repository=self._auth_repository,
                password_hasher=self._password_hasher,
            )

            user = await register_usecase.execute(register_dto)

            # Promouvoir en superuser - cela donne tous les droits
            # sans avoir besoin d'être assigné à un tenant
            user.is_superuser = True
            user.is_verified = True
            await self._auth_repository.update_user(user)

            logger.info(f"Admin par défaut créé: {self._config.default_admin_username}")

        except Exception as e:
            logger.warning(f"Impossible de créer l'admin par défaut: {e}")

    async def close(self) -> None:
        """
        Ferme toutes les connexions.

        Cette méthode doit être appelée à l'arrêt de l'application,
        typiquement dans l'événement shutdown de FastAPI.
        """
        logger.info("Fermeture de ACLManager...")

        if self._cache:
            await self._cache.disconnect()

        if self._database:
            await self._database.disconnect()

        self._initialized = False
        logger.info("ACLManager fermé")

    # ============================================
    # Getters publics
    # ============================================

    @property
    def config(self) -> ACLConfig:
        """Retourne la configuration."""
        return self._config

    @property
    def is_initialized(self) -> bool:
        """Retourne True si le manager est initialisé."""
        return self._initialized

    
    async def assign_role(
        self,
        *,
        user_id: str,
        tenant_id: str,
        role_id: str | None = None,
        role_name: str | None = None,
        assigned_by: str | None = None,
    ) -> None:
        """
        Assigne un rôle à un utilisateur dans un tenant (crée un membership).

        En mode SaaS, un utilisateur peut appartenir à plusieurs tenants
        avec des rôles différents dans chacun.

        Args:
            user_id: ID de l'utilisateur
            tenant_id: ID du tenant (fourni par l'app hôte)
            role_id: ID du rôle (optionnel si role_name fourni)
            role_name: Nom du rôle (optionnel si role_id fourni)
            assigned_by: ID de l'utilisateur qui fait l'assignation (optionnel)

        Raises:
            ConfigurationError: Si les rôles ne sont pas activés
            ValueError: Si ni role_id ni role_name n'est fourni
        """
        if not self._role_repository:
            raise ConfigurationError("Roles non initialisés. Activez enable_roles_feature=True")

        if role_id is None:
            if not role_name:
                raise ValueError("role_id ou role_name est requis")
            role = await self._role_repository.get_by_name(role_name)
            if not role:
                raise ValueError(f"Rôle introuvable: {role_name}")
            role_id = role.id

        await self._role_repository.assign_role_to_user(
            user_id=user_id,
            role_id=role_id,
            tenant_id=tenant_id,
            assigned_by=assigned_by,
        )

    async def create_account(
        self,
        username: str,
        email: str,
        password: str,
    ):
        """
        Crée un utilisateur sans passer par les routes API publiques.

        Cette méthode est conçue pour être appelée par l'application hôte
        lors de la création d'un nouveau compte utilisateur.

        Note SaaS:
            L'utilisateur n'est PAS automatiquement assigné à un tenant.
            Utilisez assign_role() après create_account() pour ajouter
            l'utilisateur à un tenant avec un rôle.

        Args:
            username: Nom d'utilisateur (globalement unique)
            email: Email (globalement unique)
            password: Mot de passe en clair

        Returns:
            AuthUser: L'utilisateur créé

        Example:
            ```python
            # 1. Créer l'utilisateur
            user = await acl.create_account(
                username="john",
                email="john@example.com",
                password="secret123",
            )

            # 2. Assigner à un tenant avec un rôle
            await acl.assign_role(
                user_id=user.id,
                tenant_id="tenant-123",
                role_name="user",
            )
            ```
        """
        if not self._auth_repository or not self._password_hasher:
            raise ConfigurationError("Auth non initialisé. Appelez ACLManager.initialize()")

        register_usecase = RegisterUseCase(
            auth_repository=self._auth_repository,
            password_hasher=self._password_hasher,
        )

        dto = RegisterDTO(username=username, email=email, password=password)
        user = await register_usecase.execute(dto)

        return user

    async def add_user_to_tenant(
        self,
        user_id: str,
        tenant_id: str,
        role_name: str = "user",
    ) -> None:
        """
        Raccourci pour ajouter un utilisateur à un tenant avec un rôle.

        Equivalent à assign_role() mais plus explicite pour l'onboarding.

        Args:
            user_id: ID de l'utilisateur
            tenant_id: ID du tenant
            role_name: Nom du rôle (défaut: "user")
        """
        await self.assign_role(
            user_id=user_id,
            tenant_id=tenant_id,
            role_name=role_name,
        )

    async def get_user_tenants(self, user_id: str) -> List[str]:
        """
        Récupère la liste des tenants auxquels un utilisateur appartient.

        Args:
            user_id: ID de l'utilisateur

        Returns:
            Liste des IDs de tenants
        """
        if not self._role_repository:
            raise ConfigurationError("Roles non initialisés. Activez enable_roles_feature=True")

        return await self._role_repository.get_user_tenants(user_id)
    
    def get_auth_repository(self) -> IAuthRepository:
        """
        Retourne le repository d'authentification.

        Returns:
            Repository d'authentification

        Raises:
            ConfigurationError: Si non initialisé
        """
        if not self._auth_repository:
            raise ConfigurationError("Auth repository non initialisé")
        return self._auth_repository

    def get_token_service(self) -> ITokenService:
        """
        Retourne le service de tokens.

        Returns:
            Service de tokens

        Raises:
            ConfigurationError: Si non initialisé
        """
        if not self._token_service:
            raise ConfigurationError("Token service non initialisé")
        return self._token_service

    def get_password_hasher(self) -> IPasswordHasher:
        """
        Retourne le service de hashage.

        Returns:
            Service de hashage

        Raises:
            ConfigurationError: Si non initialisé
        """
        if not self._password_hasher:
            raise ConfigurationError("Password hasher non initialisé")
        return self._password_hasher

    def get_database(self) -> DatabaseType:
        """
        Retourne la connexion base de données.

        Returns:
            Connexion DB

        Raises:
            ConfigurationError: Si non initialisé
        """
        if not self._database:
            raise ConfigurationError("Database non initialisée")
        return self._database

    def get_cache(self) -> CacheType:
        """
        Retourne le backend de cache.

        Returns:
            Backend de cache

        Raises:
            ConfigurationError: Si non initialisé
        """
        if not self._cache:
            raise ConfigurationError("Cache non initialisé")
        return self._cache

    def get_role_repository(self) -> IRoleRepository:
        """
        Retourne le repository des rôles.

        Returns:
            Repository des rôles

        Raises:
            ConfigurationError: Si non initialisé ou feature désactivée
        """
        if not self._role_repository:
            raise ConfigurationError(
                "Role repository non initialisé. "
                "Vérifiez que enable_roles_feature=True"
            )
        return self._role_repository

    def get_permission_repository(self) -> IPermissionRepository:
        """
        Retourne le repository des permissions.

        Returns:
            Repository des permissions

        Raises:
            ConfigurationError: Si non initialisé ou feature désactivée
        """
        if not self._permission_repository:
            raise ConfigurationError(
                "Permission repository non initialisé. "
                "Vérifiez que enable_permissions_feature=True"
            )
            
        return self._permission_repository








"""
Interfaces (ports) de la couche Application.
"""

from alak_acl.auth.application.interface.auth_repository import IAuthRepository
from alak_acl.auth.application.interface.token_service import ITokenService
from alak_acl.auth.application.interface.password_hasher import IPasswordHasher

__all__ = [
    "IAuthRepository",
    "ITokenService",
    "IPasswordHasher",
]



"""
Use Cases de la feature Auth.
"""

from alak_acl.auth.application.usecases.login_usecase import LoginUseCase
from alak_acl.auth.application.usecases.register_usecase import RegisterUseCase
from alak_acl.auth.application.usecases.refresh_token_usecase import RefreshTokenUseCase

__all__ = [
    "LoginUseCase",
    "RegisterUseCase",
    "RefreshTokenUseCase",
]



"""
Connexion MongoDB asynchrone avec motor.
"""

from typing import Any, Optional
from motor.motor_asyncio import AsyncIOMotorClient, AsyncIOMotorDatabase

from .base import BaseDatabase
from ..exceptions import DatabaseConnectionError
from ..logging import logger


class MongoDBDatabase(BaseDatabase):
    def __init__(self, uri: str, database_name: Optional[str] = None):
        self.uri = uri
        self._database_name = database_name or self._extract_db_name(uri)
        self._client: Optional[AsyncIOMotorClient] = None
        self._db: Optional[AsyncIOMotorDatabase] = None

    def _extract_db_name(self, uri: str) -> str:
        if "/" in uri.split("://")[-1]:
            parts = uri.split("/")
            db_part = parts[-1].split("?")[0]
            if db_part:
                return db_part
        return "acl_db"

    async def connect(self) -> None:
        try:
            logger.info(f"Connexion à MongoDB: {self._database_name}")
            self._client = AsyncIOMotorClient(self.uri)
            self._db = self._client[self._database_name]

            await self._client.admin.command("ping")
            logger.info("Connexion MongoDB établie avec succès")

        except Exception as e:
            logger.error(f"Erreur de connexion MongoDB: {e}")
            raise DatabaseConnectionError(f"Impossible de se connecter à MongoDB: {e}")

    async def disconnect(self) -> None:
        if self._client is not None:
            self._client.close()
            self._client = None
            self._db = None
            logger.info("Connexion MongoDB fermée")

    async def is_connected(self) -> bool:
        if self._client is None:
            return False
        try:
            await self._client.admin.command("ping")
            return True
        except Exception:
            return False

    def get_session(self) -> AsyncIOMotorDatabase:
        if self._db is None:
            raise DatabaseConnectionError("Non connecté à MongoDB")
        return self._db

    @property
    def client(self) -> AsyncIOMotorClient:
        if self._client is None:
            raise DatabaseConnectionError("Non connecté à MongoDB")
        return self._client

    @property
    def db(self) -> AsyncIOMotorDatabase:
        return self.get_session()

    @property
    def db_type(self) -> str:
        return "mongodb"

    def get_collection(self, name: str) -> Any:
        return self.db[name]



"""
Feature Roles - Gestion des rôles et permissions.

Cette feature gère les rôles, leurs associations avec les permissions,
et l'assignation des rôles aux utilisateurs.

Example:
    ```python
    from fastapi import FastAPI, Depends
    from alak_acl import ACLManager, ACLConfig
    from alak_acl.roles import RequireRole, RequirePermission

    app = FastAPI()

    config = ACLConfig(
        database_type="postgresql",
        postgresql_uri="postgresql+asyncpg://user:pass@localhost/db",
        enable_roles_feature=True,
    )

    acl = ACLManager(config, app=app)

    @app.get("/admin/dashboard")
    async def admin_dashboard(
        user = Depends(RequireRole("admin"))
    ):
        return {"message": "Bienvenue admin!"}

    @app.post("/posts")
    async def create_post(
        user = Depends(RequirePermission("posts:create"))
    ):
        return {"message": "Post créé!"}
    ```
"""

from typing import TYPE_CHECKING

# Type hints pour l'autocomplétion IDE (non exécuté à runtime)
if TYPE_CHECKING:
    from alak_acl.roles.infrastructure.models.sql_model import SQLRoleModel as SQLRoleModel
    from alak_acl.roles.infrastructure.models.sql_model import SQLUserRoleModel as SQLUserRoleModel
    from alak_acl.roles.infrastructure.models.sql_model import SQLMembershipModel as SQLMembershipModel
    from alak_acl.roles.infrastructure.models.mongo_model import MongoRoleModel as MongoRoleModel
    from alak_acl.roles.infrastructure.models.mongo_model import MongoUserRoleModel as MongoUserRoleModel
    from alak_acl.roles.infrastructure.repositories.postgresql_repository import PostgreSQLRoleRepository as PostgreSQLRoleRepository
    from alak_acl.roles.infrastructure.repositories.mysql_repository import MySQLRoleRepository as MySQLRoleRepository
    from alak_acl.roles.infrastructure.repositories.mongodb_repository import MongoDBRoleRepository as MongoDBRoleRepository
    from alak_acl.roles.interface.routes import router as router
    from alak_acl.roles.interface.dependencies import (
        set_role_dependencies as set_role_dependencies,
        get_role_repository as get_role_repository,
        get_current_user_roles as get_current_user_roles,
        get_current_user_permissions as get_current_user_permissions,
        RequireRole as RequireRole,
        RequireRoles as RequireRoles,
        RequirePermission as RequirePermission,
        RequirePermissions as RequirePermissions,
    )

# Domain
from alak_acl.roles.domain import (
    Role,
    CreateRoleDTO,
    UpdateRoleDTO,
    RoleResponseDTO,
    RoleListResponseDTO,
    AssignRoleDTO,
    AssignRolesDTO,
    UserRolesResponseDTO,
)

# Application - Interface
from alak_acl.roles.application.interface.role_repository import IRoleRepository

# Application - Use Cases
from alak_acl.roles.application.usecases import (
    CreateRoleUseCase,
    UpdateRoleUseCase,
    DeleteRoleUseCase,
    GetRoleUseCase,
    ListRolesUseCase,
    AssignRoleUseCase,
    RemoveRoleUseCase,
    GetUserRolesUseCase,
    GetUserPermissionsUseCase,
    CheckPermissionUseCase,
    CheckRoleUseCase,
    SetUserRolesUseCase,
    AssignDefaultRolesUseCase,
)

# Infrastructure - imports directs uniquement pour le mapper (pas de dépendance externe)
from alak_acl.roles.infrastructure.mappers.role_mapper import RoleMapper


# Lazy imports pour éviter les imports circulaires et les dépendances manquantes
def __getattr__(name: str):
    """Lazy loading pour éviter les imports circulaires et dépendances manquantes."""
    # Modèles MongoDB (nécessite motor/pymongo)
    if name == "MongoRoleModel":
        from alak_acl.roles.infrastructure.models.mongo_model import MongoRoleModel
        return MongoRoleModel
    elif name == "MongoUserRoleModel":
        from alak_acl.roles.infrastructure.models.mongo_model import MongoUserRoleModel
        return MongoUserRoleModel
    # Interface (Routes et dépendances) - lazy pour éviter import circulaire avec auth
    elif name == "router":
        from alak_acl.roles.interface.routes import router
        return router
    elif name == "set_role_dependencies":
        from alak_acl.roles.interface.dependencies import set_role_dependencies
        return set_role_dependencies
    elif name == "get_role_repository":
        from alak_acl.roles.interface.dependencies import get_role_repository
        return get_role_repository
    elif name == "get_current_user_roles":
        from alak_acl.roles.interface.dependencies import get_current_user_roles
        return get_current_user_roles
    elif name == "get_current_user_permissions":
        from alak_acl.roles.interface.dependencies import get_current_user_permissions
        return get_current_user_permissions
    elif name == "RequireRole":
        from alak_acl.roles.interface.dependencies import RequireRole
        return RequireRole
    elif name == "RequireRoles":
        from alak_acl.roles.interface.dependencies import RequireRoles
        return RequireRoles
    elif name == "RequirePermission":
        from alak_acl.roles.interface.dependencies import RequirePermission
        return RequirePermission
    elif name == "RequirePermissions":
        from alak_acl.roles.interface.dependencies import RequirePermissions
        return RequirePermissions
    # Modèles SQL
    elif name == "SQLRoleModel":
        from alak_acl.roles.infrastructure.models.sql_model import SQLRoleModel
        return SQLRoleModel
    elif name == "SQLUserRoleModel":
        from alak_acl.roles.infrastructure.models.sql_model import SQLUserRoleModel
        return SQLUserRoleModel
    # Repositories
    elif name == "PostgreSQLRoleRepository":
        from alak_acl.roles.infrastructure.repositories.postgresql_repository import PostgreSQLRoleRepository
        return PostgreSQLRoleRepository
    elif name == "MySQLRoleRepository":
        from alak_acl.roles.infrastructure.repositories.mysql_repository import MySQLRoleRepository
        return MySQLRoleRepository
    elif name == "MongoDBRoleRepository":
        from alak_acl.roles.infrastructure.repositories.mongodb_repository import MongoDBRoleRepository
        return MongoDBRoleRepository
    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")


__all__ = [
    # Domain - Entité
    "Role",
    # Domain - DTOs
    "CreateRoleDTO",
    "UpdateRoleDTO",
    "RoleResponseDTO",
    "RoleListResponseDTO",
    "AssignRoleDTO",
    "AssignRolesDTO",
    "UserRolesResponseDTO",
    # Application - Interface
    "IRoleRepository",
    # Application - Use Cases
    "CreateRoleUseCase",
    "UpdateRoleUseCase",
    "DeleteRoleUseCase",
    "GetRoleUseCase",
    "ListRolesUseCase",
    "AssignRoleUseCase",
    "RemoveRoleUseCase",
    "GetUserRolesUseCase",
    "GetUserPermissionsUseCase",
    "CheckPermissionUseCase",
    "CheckRoleUseCase",
    "SetUserRolesUseCase",
    "AssignDefaultRolesUseCase",
    # Infrastructure - Repositories
    "PostgreSQLRoleRepository",
    "MySQLRoleRepository",
    "MongoDBRoleRepository",
    # Infrastructure - Modèles
    "SQLRoleModel",
    "SQLUserRoleModel",
    "MongoRoleModel",
    "MongoUserRoleModel",
    # Infrastructure - Mapper
    "RoleMapper",
    # Interface - Router
    "router",
    # Interface - Configuration
    "set_role_dependencies",
    "get_role_repository",
    # Interface - Dépendances
    "get_current_user_roles",
    "get_current_user_permissions",
    "RequireRole",
    "RequireRoles",
    "RequirePermission",
    "RequirePermissions",
]



"""
Utilitaires pour la gestion du cache ACL.

Fournit des fonctions et décorateurs pour gérer le cache
de manière cohérente dans tout le package.
"""

from enum import Enum
from functools import wraps
import hashlib
import json
import inspect
from typing import Any, Callable, Optional

from pydantic import BaseModel

from alak_acl.shared.logging import logger


class CachePrefix(str, Enum):
    """Préfixes de cache pour les différentes entités ACL."""
    USER_ME = "user_me"
    USER_ROLES = "user_roles"
    USER_TENANTS = "user_tenants"
    ROLE = "role"
    PERMISSION = "permission"


def generate_hash(data: dict) -> str:
    """
    Génère un hash court à partir d'un dictionnaire.

    Args:
        data: Dictionnaire à hasher

    Returns:
        Hash MD5 tronqué (8 caractères)
    """
    json_str = json.dumps(data, sort_keys=True, ensure_ascii=False)
    return hashlib.md5(json_str.encode()).hexdigest()[:8]


def serialize_params(params: dict) -> dict:
    """
    Nettoie et sérialise les paramètres pour le cache.

    Args:
        params: Paramètres de la fonction

    Returns:
        Dictionnaire sérialisable
    """
    serializable = {}

    for key, value in params.items():
        # Ignorer les dépendances FastAPI et paramètres techniques
        if key.endswith("_usecase") or key.endswith("_repository"):
            continue
        if key in ("current_user", "config", "db", "cache"):
            continue
        if key.startswith("_"):
            continue

        # Pydantic v2
        if hasattr(value, "model_dump"):
            serializable.update(value.model_dump(mode="json"))
            continue

        # Types simples
        if isinstance(value, (str, int, float, bool, type(None))):
            serializable[key] = value
            continue

        # List / Dict / Tuple
        if isinstance(value, (list, dict, tuple)):
            serializable[key] = json.loads(
                json.dumps(value, default=str)
            )

    return serializable


def build_cache_key(
    prefix: CachePrefix,
    user_id: Optional[str] = None,
    tenant_id: Optional[str] = None,
    params: Optional[dict] = None,
) -> str:
    """
    Génère une clé de cache cohérente.

    Format: ALAKACL:{prefix}:{user_id}:{tenant_id}:{params_hash}

    Args:
        prefix: Préfixe de cache (CachePrefix enum)
        user_id: ID de l'utilisateur (optionnel)
        tenant_id: ID du tenant (optionnel)
        params: Paramètres additionnels (optionnel)

    Returns:
        Clé de cache formatée
    """
    parts = ["ALAKACL", prefix.value]

    if user_id:
        parts.append(user_id)

    if tenant_id:
        parts.append(f"tenant:{tenant_id}")
    else:
        parts.append("global")

    if params:
        params_hash = generate_hash(params)
        parts.append(params_hash)

    return ":".join(parts)


async def invalidate_user_cache(
    user_id: str,
    tenant_id: Optional[str] = None,
    prefix: CachePrefix = CachePrefix.USER_ME,
) -> None:
    """
    Invalide le cache d'un utilisateur.

    Args:
        user_id: ID de l'utilisateur
        tenant_id: ID du tenant (si None, invalide tous les tenants)
        prefix: Préfixe de cache à invalider
    """
    try:
        from alak_acl.shared.cache.factory import get_cache
        cache = get_cache()

        if tenant_id:
            # Invalider uniquement pour ce tenant
            pattern = f"ALAKACL:{prefix.value}:{user_id}:tenant:{tenant_id}*"
        else:
            # Invalider tous les tenants de cet utilisateur
            pattern = f"ALAKACL:{prefix.value}:{user_id}:*"

        deleted = await cache.scan_and_delete(pattern)
        logger.info(f"[CACHE INVALIDATE] {prefix.value} - User: {user_id} - {deleted} clés supprimées")

    except Exception as e:
        logger.warning(f"Erreur invalidation cache user {user_id}: {e}")


async def set_user_cache(
    user_id: str,
    data: Any,
    tenant_id: Optional[str] = None,
    prefix: CachePrefix = CachePrefix.USER_ME,
    ttl: Optional[int] = None,
) -> None:
    """
    Stocke des données utilisateur en cache.

    Args:
        user_id: ID de l'utilisateur
        data: Données à mettre en cache
        tenant_id: ID du tenant (optionnel)
        prefix: Préfixe de cache
        ttl: Durée de vie en secondes
    """
    try:
        from alak_acl.shared.cache.factory import get_cache
        cache = get_cache()

        cache_key = build_cache_key(prefix, user_id, tenant_id)

        # Sérialiser si Pydantic
        cache_value = data
        if hasattr(data, "model_dump"):
            cache_value = data.model_dump(mode="json")
        elif hasattr(data, "dict"):
            cache_value = data.dict()

        await cache.set(cache_key, cache_value, ttl=ttl)
        logger.debug(f"[CACHE SET] {prefix.value} - User: {user_id}")

    except Exception as e:
        logger.warning(f"Erreur écriture cache user {user_id}: {e}")


async def get_user_cache(
    user_id: str,
    tenant_id: Optional[str] = None,
    prefix: CachePrefix = CachePrefix.USER_ME,
) -> Optional[Any]:
    """
    Récupère des données utilisateur du cache.

    Args:
        user_id: ID de l'utilisateur
        tenant_id: ID du tenant (optionnel)
        prefix: Préfixe de cache

    Returns:
        Données en cache ou None
    """
    try:
        from alak_acl.shared.cache.factory import get_cache
        cache = get_cache()

        cache_key = build_cache_key(prefix, user_id, tenant_id)
        cached = await cache.get(cache_key)

        if cached:
            logger.debug(f"[CACHE HIT] {prefix.value} - User: {user_id}")
        else:
            logger.debug(f"[CACHE MISS] {prefix.value} - User: {user_id}")

        return cached

    except Exception as e:
        logger.warning(f"Erreur lecture cache user {user_id}: {e}")
        return None


async def invalidate_all_user_caches(user_id: str, tenant_id: Optional[str] = None) -> None:
    """
    Invalide tous les caches liés à un utilisateur (USER_ME, USER_ROLES, USER_TENANTS).

    Args:
        user_id: ID de l'utilisateur
        tenant_id: ID du tenant (si None, invalide tous les tenants)
    """
    for prefix in [CachePrefix.USER_ME, CachePrefix.USER_ROLES, CachePrefix.USER_TENANTS]:
        await invalidate_user_cache(user_id, tenant_id, prefix)


async def set_cache(
    key: str,
    data: Any,
    ttl: Optional[int] = None,
) -> None:
    """
    Stocke des données en cache avec une clé personnalisée.

    Args:
        key: Clé de cache
        data: Données à mettre en cache
        ttl: Durée de vie en secondes
    """
    try:
        from alak_acl.shared.cache.factory import get_cache
        cache = get_cache()

        # Sérialiser si Pydantic ou liste de Pydantic
        cache_value = data
        if hasattr(data, "model_dump"):
            cache_value = data.model_dump(mode="json")
        elif isinstance(data, list) and data and hasattr(data[0], "model_dump"):
            cache_value = [item.model_dump(mode="json") for item in data]

        await cache.set(key, cache_value, ttl=ttl)
        logger.debug(f"[CACHE SET] {key}")

    except Exception as e:
        logger.warning(f"Erreur écriture cache {key}: {e}")


async def get_cache_value(key: str) -> Optional[Any]:
    """
    Récupère des données du cache avec une clé personnalisée.

    Args:
        key: Clé de cache

    Returns:
        Données en cache ou None
    """
    try:
        from alak_acl.shared.cache.factory import get_cache
        cache = get_cache()

        cached = await cache.get(key)

        if cached:
            logger.debug(f"[CACHE HIT] {key}")
        else:
            logger.debug(f"[CACHE MISS] {key}")

        return cached

    except Exception as e:
        logger.warning(f"Erreur lecture cache {key}: {e}")
        return None


async def invalidate_cache_pattern(pattern: str) -> int:
    """
    Invalide toutes les clés correspondant au pattern.

    Args:
        pattern: Pattern de recherche (ex: "ALAKACL:role:*")

    Returns:
        Nombre de clés supprimées
    """
    try:
        from alak_acl.shared.cache.factory import get_cache
        cache = get_cache()

        deleted = await cache.scan_and_delete(pattern)
        logger.info(f"[CACHE INVALIDATE] Pattern: {pattern} - {deleted} clés supprimées")
        return deleted

    except Exception as e:
        logger.warning(f"Erreur invalidation cache pattern {pattern}: {e}")
        return 0



"""
Repository MongoDB pour l'authentification.

Supporte les champs personnalisés ajoutés directement aux documents.
"""

from typing import Any, Optional, List, Type

from bson import ObjectId


from alak_acl.auth.application.interface.auth_repository import IAuthRepository
from alak_acl.auth.domain.entities.auth_user import AuthUser
from alak_acl.auth.infrastructure.mappers.auth_user_mapper import AuthUserMapper
from alak_acl.auth.infrastructure.models.mongo_model import MongoAuthUserModel
from alak_acl.shared.database.mongodb import MongoDBDatabase
from alak_acl.shared.exceptions import UserAlreadyExistsError, UserNotFoundError
from alak_acl.shared.logging import logger


class MongoDBAuthRepository(IAuthRepository):
    """
    Implémentation du repository d'authentification pour MongoDB.

    Utilise motor pour les opérations asynchrones.
    Supporte les champs personnalisés qui sont ajoutés directement aux documents.

    Attributes:
        db: Instance de connexion MongoDB
        collection_name: Nom de la collection
        model_class: Classe du modèle Pydantic (optionnel)
        mapper: Instance du mapper

    Example:
        ```python
        # Avec modèle personnalisé
        from typing import Optional
        from pydantic import Field
        from alak_acl.auth.infrastructure.models import MongoAuthUserModel

        class CustomUserModel(MongoAuthUserModel):
            phone: Optional[str] = Field(None, max_length=20)
            company: Optional[str] = Field(None, max_length=100)

        repo = MongoDBAuthRepository(
            db=database,
            collection_name="users",
            model_class=CustomUserModel,
        )
        ```
    """

    def __init__(
        self,
        db: MongoDBDatabase,
        collection_name: str = "auth_users",
        model_class: Type[MongoAuthUserModel] = MongoAuthUserModel,
        mapper: Optional[AuthUserMapper] = None,
    ):
        """
        Initialise le repository.

        Args:
            db: Instance de MongoDBDatabase
            collection_name: Nom de la collection MongoDB
            model_class: Classe du modèle Pydantic (par défaut ou personnalisée)
            mapper: Instance du mapper personnalisée (optionnel)
        """
        self._db = db
        self._collection_name = collection_name
        self._model_class = model_class
        self._mapper = mapper or AuthUserMapper(mongo_model_class=model_class)

    @property
    def _collection(self):
        """Retourne la collection MongoDB."""
        return self._db.get_collection(self._collection_name)

    @property
    def collection_name(self) -> str:
        """Retourne le nom de la collection."""
        return self._collection_name

    @property
    def model_class(self) -> Type[MongoAuthUserModel]:
        """Retourne la classe du modèle utilisé."""
        return self._model_class

    async def create_user(self, user: AuthUser) -> AuthUser:
        """
        Crée un nouvel utilisateur.

        Les champs personnalisés dans extra_fields sont automatiquement
        ajoutés au document MongoDB. L'_id est généré automatiquement par MongoDB.
        """
        # Vérifier l'unicité
        existing = await self._collection.find_one({
            "$or": [
                {"username": user.username},
                {"email": user.email}
            ]
        })
        if existing:
            raise UserAlreadyExistsError(
                "__all__",
                "Un utilisateur avec ce username ou email existe déjà"
            )

        doc = self._mapper.to_mongo_dict(user)
        # Ne pas inclure _id - MongoDB le génère automatiquement
        doc.pop("_id", None)

        result = await self._collection.insert_one(doc)
        # Récupérer l'ID généré par MongoDB
        user.id = str(result.inserted_id)

        logger.debug(f"Utilisateur créé: {user.username} (id: {user.id})")
        return user

    async def get_by_id(self, user_id: str) -> Optional[AuthUser]:
        """Récupère un utilisateur par son ID (ObjectId string)."""
        try:
            doc = await self._collection.find_one({"_id": ObjectId(user_id)})
            if doc:
                return self._mapper.to_entity(doc)
        except Exception:
            pass
        return None

    async def get_by_username(self, username: str) -> Optional[AuthUser]:
        """Récupère un utilisateur par son username."""
        doc = await self._collection.find_one({"username": username})
        if doc:
            return self._mapper.to_entity(doc)
        return None

    async def get_by_email(self, email: str) -> Optional[AuthUser]:
        """Récupère un utilisateur par son email."""
        doc = await self._collection.find_one({"email": email})
        if doc:
            return self._mapper.to_entity(doc)
        return None

    async def update_user(self, user: AuthUser) -> AuthUser:
        """
        Met à jour un utilisateur.

        Les champs personnalisés dans extra_fields sont automatiquement
        mis à jour dans le document MongoDB.
        """
        doc = self._mapper.to_mongo_dict(user)
        result = await self._collection.update_one(
            {"_id": ObjectId(user.id)},
            {'$set': doc}
        )

        if result.matched_count == 0:
            raise UserNotFoundError("id", f"Utilisateur non trouvé: {user.id}")

        logger.debug(f"Utilisateur mis à jour: {user.username}")
        return user

    async def delete_user(self, user_id: str) -> bool:
        """Supprime un utilisateur par son ID (ObjectId string)."""
        try:
            result = await self._collection.delete_one({"_id": ObjectId(user_id)})
            if result.deleted_count > 0:
                logger.debug(f"Utilisateur supprimé: {user_id}")
                return True
        except Exception:
            pass
        return False

    async def list_users(
        self,
        skip: int = 0,
        limit: int = 100,
        is_active: Optional[bool] = None,
    ) -> List[AuthUser]:
        """Liste les utilisateurs avec pagination."""
        query = {}
        if is_active is not None:
            query["is_active"] = is_active

        cursor = self._collection.find(query).skip(skip).limit(limit).sort("created_at", -1)
        docs = await cursor.to_list(length=limit)

        return [self._mapper.to_entity(doc) for doc in docs]

    async def count_users(self, is_active: Optional[bool] = None) -> int:
        """Compte le nombre d'utilisateurs."""
        query = {}
        if is_active is not None:
            query["is_active"] = is_active

        return await self._collection.count_documents(query)

    async def username_exists(self, username: str) -> bool:
        """Vérifie si un username existe."""
        count = await self._collection.count_documents({"username": username})
        return count > 0

    async def email_exists(self, email: str) -> bool:
        """Vérifie si un email existe."""
        count = await self._collection.count_documents({"email": email})
        return count > 0

    async def find_by_extra_field(
        self,
        field_name: str,
        value: Any,
    ) -> Optional[AuthUser]:
        """
        Recherche un utilisateur par un champ personnalisé.

        Args:
            field_name: Nom du champ personnalisé
            value: Valeur à rechercher

        Returns:
            Utilisateur trouvé ou None
        """
        doc = await self._collection.find_one({field_name: value})
        if doc:
            return self._mapper.to_entity(doc)
        return None

    async def list_by_extra_field(
        self,
        field_name: str,
        value: Any,
        skip: int = 0,
        limit: int = 100,
    ) -> List[AuthUser]:
        """
        Liste les utilisateurs par un champ personnalisé.

        Args:
            field_name: Nom du champ personnalisé
            value: Valeur à rechercher
            skip: Offset pour la pagination
            limit: Limite du nombre de résultats

        Returns:
            Liste d'utilisateurs correspondants
        """
        cursor = (
            self._collection
            .find({field_name: value})
            .skip(skip)
            .limit(limit)
            .sort("created_at", -1)
        )
        docs = await cursor.to_list(length=limit)

        return [self._mapper.to_entity(doc) for doc in docs]

    async def create_indexes(self, extra_indexes: Optional[List[str]] = None) -> None:
        """
        Crée les index nécessaires pour optimiser les requêtes.

        Args:
            extra_indexes: Liste de noms de champs personnalisés à indexer

        À appeler lors de l'initialisation.
        """
        # Index composites uniques : username et email uniques par tenant
        await self._collection.create_index(
            [("tenant_id", 1), ("username", 1)],
            unique=True,
            name="uq_user_tenant_username"
        )
        await self._collection.create_index(
            [("tenant_id", 1), ("email", 1)],
            unique=True,
            name="uq_user_tenant_email"
        )
        # Index simples pour les recherches
        # await self._collection.create_index("username")
        # await self._collection.create_index("email")
        # await self._collection.create_index("tenant_id")
        await self._collection.create_index("is_active")
        await self._collection.create_index("created_at")

        # Index sur les champs personnalisés
        if extra_indexes:
            for field_name in extra_indexes:
                await self._collection.create_index(field_name)
                logger.debug(f"Index créé pour le champ personnalisé: {field_name}")

        logger.info(f"Index MongoDB créés pour {self._collection_name}")

    async def update_extra_field(
        self,
        user_id: str,
        field_name: str,
        value: Any,
    ) -> bool:
        """
        Met à jour un seul champ personnalisé.

        Args:
            user_id: ID de l'utilisateur
            field_name: Nom du champ à mettre à jour
            value: Nouvelle valeur

        Returns:
            True si la mise à jour a réussi
        """
        result = await self._collection.update_one(
            {"_id": ObjectId(user_id)},
            {"$set": {field_name: value}}
        )
        return result.modified_count > 0

    async def add_to_array_field(
        self,
        user_id: str,
        field_name: str,
        value: Any,
    ) -> bool:
        """
        Ajoute une valeur à un champ de type array.

        Args:
            user_id: ID de l'utilisateur
            field_name: Nom du champ array
            value: Valeur à ajouter

        Returns:
            True si l'ajout a réussi
        """
        result = await self._collection.update_one(
            {"_id": ObjectId(user_id)},
            {"$push": {field_name: value}}
        )
        return result.modified_count > 0

    async def remove_from_array_field(
        self,
        user_id: str,
        field_name: str,
        value: Any,
    ) -> bool:
        """
        Retire une valeur d'un champ de type array.

        Args:
            user_id: ID de l'utilisateur
            field_name: Nom du champ array
            value: Valeur à retirer

        Returns:
            True si la suppression a réussi
        """
        result = await self._collection.update_one(
            {"_id": ObjectId(user_id)},
            {"$pull": {field_name: value}}
        )
        return result.modified_count > 0



"""
fastapi-acl - Package de gestion ACL pour FastAPI.

Package professionnel pour gérer l'authentification et les permissions
(ACL - Access Control List) dans des applications FastAPI.

Example:
    ```python
    from fastapi import FastAPI
    from alak_acl import ACLManager, ACLConfig

    app = FastAPI()

    config = ACLConfig(
        database_type="postgresql",
        postgresql_uri="postgresql+asyncpg://user:pass@localhost/db",
        enable_cache=True,
        redis_url="redis://localhost:6379/0",
        enable_auth_feature=True,
        enable_api_routes=True,
    )

    acl = ACLManager(config, app=app)

    @app.on_event("startup")
    async def startup():
        await acl.initialize()

    @app.on_event("shutdown")
    async def shutdown():
        await acl.close()

    # Les routes sont automatiquement enregistrées dans Swagger !
    ```
"""

from typing import TYPE_CHECKING

__version__ = "0.1.0"
__author__ = "ALAK Team"
__license__ = "MIT"

# Type hints pour l'autocomplétion IDE (non exécuté à runtime)
if TYPE_CHECKING:
    from alak_acl.shared.database.declarative_base import Base as Base
    from alak_acl.auth.infrastructure.models.sql_model import SQLAuthUserModel as SQLAuthUserModel
    from alak_acl.auth.infrastructure.models.mongo_model import MongoAuthUserModel as MongoAuthUserModel
    from alak_acl.roles.infrastructure.models.sql_model import SQLRoleModel as SQLRoleModel
    from alak_acl.roles.infrastructure.models.sql_model import SQLUserRoleModel as SQLUserRoleModel
    from alak_acl.roles.infrastructure.models.sql_model import SQLMembershipModel as SQLMembershipModel
    from alak_acl.permissions.infrastructure.models.sql_model import SQLPermissionModel as SQLPermissionModel

from alak_acl.manager import ACLManager
from alak_acl.shared.config import ACLConfig
from alak_acl.shared.exceptions import (
    ACLException,
    AuthenticationError,
    InvalidCredentialsError,
    InvalidTokenError,
    TokenExpiredError,
    UserNotFoundError,
    UserNotActiveError,
    UserAlreadyExistsError,
    UserNotVerifiedError,
    PermissionDeniedError,
    PermissionNotFoundError,
    PermissionAlreadyExistsError,
    RoleNotFoundError,
    RoleAlreadyExistsError,
    DatabaseConnectionError,
    CacheConnectionError,
    ConfigurationError,
    ResetTokenExpiredError,
    ResetTokenInvalidError,
    EmailSendError,
)

# Entités et DTOs Auth
from alak_acl.auth.domain.entities.auth_user import AuthUser
from alak_acl.auth.domain.dtos.login_dto import LoginDTO
from alak_acl.auth.domain.dtos.register_dto import RegisterDTO
from alak_acl.auth.domain.dtos.token_dto import TokenDTO
from alak_acl.auth.domain.dtos.password_reset_dto import ForgotPasswordDTO, ResetPasswordDTO

# Mapper Auth (pas de dépendance externe)
from alak_acl.auth.infrastructure.mappers.auth_user_mapper import AuthUserMapper

# Dépendances FastAPI Auth
from alak_acl.auth.interface.dependencies import (
    get_current_user,
    get_current_active_user,
    get_current_superuser,
)

# Entités et DTOs Roles
from alak_acl.roles.domain.entities.role import Role
from alak_acl.roles.domain.dtos.role_dto import (
    CreateRoleDTO,
    UpdateRoleDTO,
    RoleResponseDTO,
    AssignRoleDTO,
    UserRolesResponseDTO,
)

# Dépendances FastAPI Roles
from alak_acl.roles.interface.dependencies import (
    RequireRole,
    RequireRoles,
    RequirePermission,
    RequirePermissions,
    get_current_user_roles,
    get_current_user_permissions,
)

# Entités et DTOs Permissions
from alak_acl.permissions.domain.entities.permission import Permission
from alak_acl.permissions.domain.dtos.permission_dto import (
    CreatePermissionDTO,
    UpdatePermissionDTO,
    PermissionResponseDTO,
    PermissionListResponseDTO,
)


# Lazy imports pour les modèles (évite de charger les dépendances non installées)
def __getattr__(name: str):
    """Lazy loading des classes pour éviter les dépendances manquantes."""
    # Modèles MongoDB Auth (nécessite motor/pymongo)
    if name == "MongoAuthUserModel":
        from alak_acl.auth.infrastructure.models.mongo_model import MongoAuthUserModel
        return MongoAuthUserModel
    # Modèles SQL Auth (nécessite SQLAlchemy)
    elif name == "SQLAuthUserModel":
        from alak_acl.auth.infrastructure.models.sql_model import SQLAuthUserModel
        return SQLAuthUserModel
    # Base SQLAlchemy
    elif name == "Base":
        from alak_acl.shared.database.declarative_base import Base
        return Base
    # Modèles SQL Roles
    elif name == "SQLRoleModel":
        from alak_acl.roles.infrastructure.models.sql_model import SQLRoleModel
        return SQLRoleModel
    elif name == "SQLUserRoleModel":
        from alak_acl.roles.infrastructure.models.sql_model import SQLUserRoleModel
        return SQLUserRoleModel
    elif name == "SQLMembershipModel":
        from alak_acl.roles.infrastructure.models.sql_model import SQLMembershipModel
        return SQLMembershipModel
    # Modèles SQL Permissions
    elif name == "SQLPermissionModel":
        from alak_acl.permissions.infrastructure.models.sql_model import SQLPermissionModel
        return SQLPermissionModel
    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")

__all__ = [
    # Version
    "__version__",
    # Manager principal
    "ACLManager",
    "ACLConfig",
    # Exceptions
    "ACLException",
    "AuthenticationError",
    "InvalidCredentialsError",
    "InvalidTokenError",
    "TokenExpiredError",
    "UserNotFoundError",
    "UserNotActiveError",
    "UserAlreadyExistsError",
    "UserNotVerifiedError",
    "PermissionDeniedError",
    "PermissionNotFoundError",
    "PermissionAlreadyExistsError",
    "RoleNotFoundError",
    "RoleAlreadyExistsError",
    "DatabaseConnectionError",
    "CacheConnectionError",
    "ConfigurationError",
    "ResetTokenExpiredError",
    "ResetTokenInvalidError",
    "EmailSendError",
    # Entités et DTOs
    "AuthUser",
    "LoginDTO",
    "RegisterDTO",
    "TokenDTO",
    "ForgotPasswordDTO",
    "ResetPasswordDTO",
    # Modèles extensibles (pour champs personnalisés)
    "SQLAuthUserModel",
    "MongoAuthUserModel",
    "AuthUserMapper",
    # Dépendances Auth
    "get_current_user",
    "get_current_active_user",
    "get_current_superuser",
    # Entités et DTOs Roles
    "Role",
    "CreateRoleDTO",
    "UpdateRoleDTO",
    "RoleResponseDTO",
    "AssignRoleDTO",
    "UserRolesResponseDTO",
    # Dépendances Roles
    "RequireRole",
    "RequireRoles",
    "RequirePermission",
    "RequirePermissions",
    "get_current_user_roles",
    "get_current_user_permissions",
    # Entités et DTOs Permissions
    "Permission",
    "CreatePermissionDTO",
    "UpdatePermissionDTO",
    "PermissionResponseDTO",
    "PermissionListResponseDTO",
    # Base SQLAlchemy et modèles pour migrations
    "Base",
    "SQLRoleModel",
    "SQLUserRoleModel",
    "SQLMembershipModel",
    "SQLPermissionModel",
]



"""
Dépendances FastAPI pour l'authentification.
"""

from typing import Optional, TYPE_CHECKING

from fastapi import Depends, HTTPException, status
from fastapi.security import HTTPBearer, HTTPAuthorizationCredentials

from alak_acl.auth.domain.entities.auth_user import AuthUser
from alak_acl.auth.application.interface.auth_repository import IAuthRepository
from alak_acl.auth.application.interface.token_service import ITokenService
from alak_acl.auth.application.interface.password_hasher import IPasswordHasher
from alak_acl.auth.application.usecases.login_usecase import LoginUseCase
from alak_acl.auth.application.usecases.register_usecase import RegisterUseCase
from alak_acl.auth.application.usecases.refresh_token_usecase import RefreshTokenUseCase
from alak_acl.auth.application.usecases.forgot_password_usecase import ForgotPasswordUseCase
from alak_acl.auth.application.usecases.reset_password_usecase import ResetPasswordUseCase
from alak_acl.auth.application.interface.email_service import IEmailService
from alak_acl.shared.config import ACLConfig
from alak_acl.shared.exceptions import (
    InvalidTokenError,
    TokenExpiredError,
)
from alak_acl.shared.logging import logger

if TYPE_CHECKING:
    from alak_acl.roles.application.interface.role_repository import IRoleRepository


# Security scheme pour Swagger
security = HTTPBearer(auto_error=False)


# Variables globales pour les instances (initialisées par ACLManager)
_auth_repository: Optional[IAuthRepository] = None
_token_service: Optional[ITokenService] = None
_password_hasher: Optional[IPasswordHasher] = None
_role_repository: Optional["IRoleRepository"] = None
_email_service: Optional[IEmailService] = None
_reset_url_base: Optional[str] = None
_config: Optional[ACLConfig] = None


def set_auth_dependencies(
    auth_repository: IAuthRepository,
    token_service: ITokenService,
    password_hasher: IPasswordHasher,
    role_repository: Optional["IRoleRepository"] = None,
    config: Optional[ACLConfig] = None,
) -> None:
    """
    Configure les dépendances d'authentification.

    Appelé par ACLManager lors de l'initialisation.

    Args:
        auth_repository: Repository d'authentification
        token_service: Service de tokens
        password_hasher: Service de hashage
        role_repository: Repository des rôles (optionnel)
        config: Configuration ACL (optionnel)
    """
    global _auth_repository, _token_service, _password_hasher, _role_repository, _config
    _auth_repository = auth_repository
    _token_service = token_service
    _password_hasher = password_hasher
    _role_repository = role_repository
    _config = config


def set_email_dependencies(
    email_service: Optional[IEmailService] = None,
    reset_url_base: Optional[str] = None,
) -> None:
    """
    Configure les dépendances email pour le reset de mot de passe.

    Appelé par ACLManager lors de l'initialisation.

    Args:
        email_service: Service d'email (SMTP ou Console)
        reset_url_base: URL de base pour les liens de reset
    """
    global _email_service, _reset_url_base
    _email_service = email_service
    _reset_url_base = reset_url_base


def get_auth_repository() -> IAuthRepository:
    """
    Récupère le repository d'authentification.

    Returns:
        Repository d'authentification

    Raises:
        HTTPException: Si non initialisé
    """
    if _auth_repository is None:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail="Service d'authentification non initialisé",
        )
    return _auth_repository


def get_token_service() -> ITokenService:
    """
    Récupère le service de tokens.

    Returns:
        Service de tokens

    Raises:
        HTTPException: Si non initialisé
    """
    if _token_service is None:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail="Service de tokens non initialisé",
        )
    return _token_service


def get_password_hasher() -> IPasswordHasher:
    """
    Récupère le service de hashage.

    Returns:
        Service de hashage

    Raises:
        HTTPException: Si non initialisé
    """
    if _password_hasher is None:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail="Service de hashage non initialisé",
        )
    return _password_hasher


def get_role_repository() -> Optional["IRoleRepository"]:
    """
    Récupère le repository des rôles.

    Returns:
        Repository des rôles ou None si non configuré
    """
    return _role_repository


def get_email_service() -> Optional[IEmailService]:
    """
    Récupère le service d'email.

    Returns:
        Service d'email ou None si non configuré
    """
    return _email_service

def get_config() -> ACLConfig:
    """
    Récupère la configuration ACL.

    Returns:
        Configuration ACL

    Raises:
        HTTPException: Si non initialisé
    """
    if _config is None:
        # Retourner une config par défaut si non initialisée
        return ACLConfig()
    return _config


def get_login_usecase(
    auth_repository: IAuthRepository = Depends(get_auth_repository),
    token_service: ITokenService = Depends(get_token_service),
    password_hasher: IPasswordHasher = Depends(get_password_hasher),
) -> LoginUseCase:
    """
    Instancie le use case de connexion.

    Returns:
        Instance de LoginUseCase
    """
    return LoginUseCase(
        auth_repository=auth_repository,
        token_service=token_service,
        password_hasher=password_hasher,
    )


def get_register_usecase(
    auth_repository: IAuthRepository = Depends(get_auth_repository),
    password_hasher: IPasswordHasher = Depends(get_password_hasher),
    role_repository: Optional["IRoleRepository"] = Depends(get_role_repository),
) -> RegisterUseCase:
    """
    Instancie le use case d'inscription.

    Le role_repository est injecté pour permettre l'assignation automatique
    du rôle par défaut lors de l'inscription.

    Returns:
        Instance de RegisterUseCase
    """
    return RegisterUseCase(
        auth_repository=auth_repository,
        password_hasher=password_hasher,
        role_repository=role_repository,
    )


def get_refresh_token_usecase(
    auth_repository: IAuthRepository = Depends(get_auth_repository),
    token_service: ITokenService = Depends(get_token_service),
) -> RefreshTokenUseCase:
    """
    Instancie le use case de rafraîchissement de token.

    Returns:
        Instance de RefreshTokenUseCase
    """
    return RefreshTokenUseCase(
        auth_repository=auth_repository,
        token_service=token_service,
    )


async def get_current_user(
    credentials: Optional[HTTPAuthorizationCredentials] = Depends(security),
    auth_repository: IAuthRepository = Depends(get_auth_repository),
    token_service: ITokenService = Depends(get_token_service),
) -> AuthUser:
    """
    Récupère l'utilisateur courant depuis le token JWT.

    Args:
        credentials: Credentials HTTP Bearer
        auth_repository: Repository d'authentification
        token_service: Service de tokens

    Returns:
        Entité AuthUser de l'utilisateur connecté

    Raises:
        HTTPException 401: Si le token est invalide ou manquant
    """
    if not credentials:
        raise HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED,
            detail="Token d'authentification requis",
            headers={"WWW-Authenticate": "Bearer"},
        )

    token = credentials.credentials

    try:
        user_id = token_service.get_user_id_from_token(token)
    except (InvalidTokenError, TokenExpiredError) as e:
        logger.warning(f"Token invalide: {e}")
        raise e

    user = await auth_repository.get_by_id(user_id)

    if not user:
        raise HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED,
            detail="Utilisateur non trouvé",
            headers={"WWW-Authenticate": "Bearer"},
        )

    return user


async def get_current_active_user(
    current_user: AuthUser = Depends(get_current_user),
) -> AuthUser:
    """
    Récupère l'utilisateur courant s'il est actif.

    Args:
        current_user: Utilisateur courant

    Returns:
        Utilisateur actif

    Raises:
        HTTPException 403: Si le compte est désactivé
    """
    if not current_user.is_active:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Compte utilisateur désactivé",
        )
    return current_user


async def get_current_superuser(
    current_user: AuthUser = Depends(get_current_active_user),
) -> AuthUser:
    """
    Récupère l'utilisateur courant s'il est superuser.

    Args:
        current_user: Utilisateur courant actif

    Returns:
        Utilisateur superuser

    Raises:
        HTTPException 403: Si l'utilisateur n'est pas superuser
    """
    if not current_user.is_superuser:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Droits administrateur requis",
        )
    return current_user


def get_forgot_password_usecase(
    auth_repository: IAuthRepository = Depends(get_auth_repository),
    token_service: ITokenService = Depends(get_token_service),
    email_service: Optional[IEmailService] = Depends(get_email_service),
) -> ForgotPasswordUseCase:
    """
    Instancie le use case de demande de réinitialisation de mot de passe.

    Returns:
        Instance de ForgotPasswordUseCase
    """
    from alak_acl.auth.infrastructure.services.console_email_service import ConsoleEmailService

    # Utiliser ConsoleEmailService si aucun service email n'est configuré
    effective_email_service = email_service or ConsoleEmailService()

    return ForgotPasswordUseCase(
        auth_repository=auth_repository,
        token_service=token_service,
        email_service=effective_email_service,
        reset_url_base=_reset_url_base,
    )


def get_reset_password_usecase(
    auth_repository: IAuthRepository = Depends(get_auth_repository),
    token_service: ITokenService = Depends(get_token_service),
    password_hasher: IPasswordHasher = Depends(get_password_hasher),
) -> ResetPasswordUseCase:
    """
    Instancie le use case de réinitialisation de mot de passe.

    Returns:
        Instance de ResetPasswordUseCase
    """
    return ResetPasswordUseCase(
        auth_repository=auth_repository,
        token_service=token_service,
        password_hasher=password_hasher,
    )



"""
Modèle Pydantic pour les documents MongoDB (Roles).

Ce module fournit un modèle de base extensible que le développeur
peut personnaliser en ajoutant ses propres champs via héritage.
"""

from datetime import datetime
from typing import List, Optional

from pydantic import BaseModel, Field, ConfigDict


class MongoRoleModel(BaseModel):
    """
    Modèle Pydantic pour les documents rôles MongoDB.

    Ce modèle peut être étendu par le développeur pour ajouter
    des champs personnalisés via héritage.

    Attributes:
        id: Identifiant unique (ObjectId généré par MongoDB)
        name: Nom unique du rôle
        display_name: Nom d'affichage
        description: Description du rôle
        permissions: Liste des permissions
        is_active: Rôle actif
        is_default: Rôle par défaut
        is_system: Rôle système
        priority: Priorité du rôle
        tenant_id: Identifiant du tenant (optionnel)
        created_at: Date de création
        updated_at: Date de mise à jour

    Example:
        Pour ajouter des champs personnalisés, créez une sous-classe:

        ```python
        from typing import Optional
        from pydantic import Field
        from alak_acl.roles.infrastructure.models import MongoRoleModel

        class CustomRoleModel(MongoRoleModel):
            department: Optional[str] = Field(None, max_length=100)
        ```
    """

    model_config = ConfigDict(
        populate_by_name=True,
        arbitrary_types_allowed=True,
    )

    id: Optional[str] = Field(default=None, alias="_id")
    name: str = Field(..., min_length=2, max_length=50)
    display_name: Optional[str] = Field(None, max_length=100)
    description: Optional[str] = Field(None, max_length=500)
    permissions: List[str] = Field(default_factory=list)
    is_active: bool = True
    is_default: bool = False
    is_system: bool = False
    priority: int = Field(default=0, ge=0)
    tenant_id: Optional[str] = None
    created_at: datetime = Field(default_factory=datetime.utcnow)
    updated_at: datetime = Field(default_factory=datetime.utcnow)

    def to_mongo_dict(self, include_id: bool = False) -> dict:
        """
        Convertit le modèle en dictionnaire pour MongoDB.

        Args:
            include_id: Si True, inclut _id dans le dictionnaire

        Returns:
            Dictionnaire compatible MongoDB
        """
        data = self.model_dump(by_alias=True, exclude_none=False)
        if not include_id or data.get("_id") is None:
            data.pop("_id", None)
        return data

    @classmethod
    def from_mongo_dict(cls, data: dict) -> "MongoRoleModel":
        """
        Crée une instance depuis un document MongoDB.

        Args:
            data: Document MongoDB

        Returns:
            Instance du modèle
        """
        if "_id" in data:
            data["_id"] = str(data["_id"])
        return cls(**data)


class MongoMembershipModel(BaseModel):
    """
    Modèle pour les memberships (pivot user-tenant-role) dans MongoDB.

    Représente l'appartenance d'un utilisateur à un tenant avec un rôle.
    C'est le cœur du système SaaS multi-tenant.

    Un utilisateur peut:
    - Appartenir à plusieurs tenants
    - Avoir plusieurs rôles dans un même tenant
    - Avoir des rôles différents selon le tenant

    Attributes:
        user_id: ID de l'utilisateur
        tenant_id: ID du tenant (fourni par l'app hôte)
        role_id: ID du rôle
        assigned_at: Date d'assignation
        assigned_by: ID de l'utilisateur ayant fait l'assignation (optionnel)
    """

    model_config = ConfigDict(
        populate_by_name=True,
    )

    id: Optional[str] = Field(default=None, alias="_id")
    user_id: str
    tenant_id: str  # Obligatoire pour SaaS
    role_id: str
    assigned_at: datetime = Field(default_factory=datetime.utcnow)
    assigned_by: Optional[str] = None

    def to_mongo_dict(self, include_id: bool = False) -> dict:
        """Convertit en dictionnaire MongoDB."""
        data = self.model_dump(by_alias=True, exclude_none=False)
        if not include_id or data.get("_id") is None:
            data.pop("_id", None)
        return data

    @classmethod
    def from_mongo_dict(cls, data: dict) -> "MongoMembershipModel":
        """Crée une instance depuis un document MongoDB."""
        if "_id" in data:
            data["_id"] = str(data["_id"])
        return cls(**data)


# Alias pour rétrocompatibilité (déprécié)
MongoUserRoleModel = MongoMembershipModel



"""
Interface du repository d'authentification.
"""

from abc import ABC, abstractmethod
from typing import Optional, List

from alak_acl.auth.domain.entities.auth_user import AuthUser



class IAuthRepository(ABC):
    """
    Interface abstraite pour le repository d'authentification.

    Définit le contrat que chaque implémentation de repository
    (MongoDB, PostgreSQL, MySQL) doit respecter.
    """

    @abstractmethod
    async def create_user(self, user: AuthUser) -> AuthUser:
        """
        Crée un nouvel utilisateur.

        Args:
            user: Entité utilisateur à créer

        Returns:
            Utilisateur créé avec son ID

        Raises:
            UserAlreadyExistsError: Si username ou email existe déjà
        """
        pass

    @abstractmethod
    async def get_by_id(self, user_id: str) -> Optional[AuthUser]:
        """
        Récupère un utilisateur par son ID.

        Args:
            user_id: ID de l'utilisateur (string)

        Returns:
            Utilisateur ou None si non trouvé
        """
        pass

    @abstractmethod
    async def get_by_username(self, username: str) -> Optional[AuthUser]:
        """
        Récupère un utilisateur par son nom d'utilisateur.

        Args:
            username: Nom d'utilisateur

        Returns:
            Utilisateur ou None si non trouvé
        """
        pass

    @abstractmethod
    async def get_by_email(self, email: str) -> Optional[AuthUser]:
        """
        Récupère un utilisateur par son email.

        Args:
            email: Adresse email

        Returns:
            Utilisateur ou None si non trouvé
        """
        pass

    @abstractmethod
    async def update_user(self, user: AuthUser) -> AuthUser:
        """
        Met à jour un utilisateur existant.

        Args:
            user: Entité utilisateur avec les modifications

        Returns:
            Utilisateur mis à jour

        Raises:
            UserNotFoundError: Si l'utilisateur n'existe pas
        """
        pass

    @abstractmethod
    async def delete_user(self, user_id: str) -> bool:
        """
        Supprime un utilisateur.

        Args:
            user_id: ID de l'utilisateur (string)

        Returns:
            True si supprimé, False si non trouvé
        """
        pass

    @abstractmethod
    async def list_users(
        self,
        skip: int = 0,
        limit: int = 100,
        is_active: Optional[bool] = None,
    ) -> List[AuthUser]:
        """
        Liste les utilisateurs avec pagination.

        Args:
            skip: Nombre d'éléments à sauter
            limit: Nombre maximum d'éléments
            is_active: Filtrer par statut actif (optionnel)

        Returns:
            Liste d'utilisateurs
        """
        pass

    @abstractmethod
    async def count_users(self, is_active: Optional[bool] = None) -> int:
        """
        Compte le nombre d'utilisateurs.

        Args:
            is_active: Filtrer par statut actif (optionnel)

        Returns:
            Nombre d'utilisateurs
        """
        pass

    @abstractmethod
    async def username_exists(self, username: str) -> bool:
        """
        Vérifie si un nom d'utilisateur existe.

        Args:
            username: Nom d'utilisateur

        Returns:
            True si existe
        """
        pass

    @abstractmethod
    async def email_exists(self, email: str) -> bool:
        """
        Vérifie si un email existe.

        Args:
            email: Adresse email

        Returns:
            True si existe
        """
        pass



"""
Couche Domain - Entités et DTOs métier.
"""

from alak_acl.auth.domain.entities.auth_user import AuthUser
from alak_acl.auth.domain.dtos.login_dto import LoginDTO
from alak_acl.auth.domain.dtos.register_dto import RegisterDTO
from alak_acl.auth.domain.dtos.token_dto import TokenDTO

__all__ = [
    "AuthUser",
    "LoginDTO",
    "RegisterDTO",
    "TokenDTO",
]



"""
Implémentation du cache avec Redis.
"""

import json
from typing import Any, Optional

import redis.asyncio as redis

from alak_acl.shared.cache.base import CacheBackend
from alak_acl.shared.exceptions import CacheConnectionError
from alak_acl.shared.logging import logger


class RedisCache(CacheBackend):
    """
    Implémentation du cache avec Redis asynchrone.

    Attributes:
        url: URL de connexion Redis
        default_ttl: TTL par défaut en secondes
        client: Client Redis
    """

    def __init__(self, url: str, default_ttl: int = 300):
        """
        Initialise le cache Redis.

        Args:
            url: URL de connexion (ex: redis://localhost:6379/0)
            default_ttl: TTL par défaut en secondes
        """
        self.url = url
        self.default_ttl = default_ttl
        self._client: Optional[redis.Redis] = None

    async def connect(self) -> None:
        """
        Établit la connexion à Redis.

        Raises:
            CacheConnectionError: Si la connexion échoue
        """
        try:
            logger.info("Connexion à Redis...")
            self._client = redis.from_url(
                self.url,
                encoding="utf-8",
                decode_responses=True,
            )
            # Test de la connexion
            await self._client.ping()
            logger.info("Connexion Redis établie avec succès")

        except Exception as e:
            logger.error(f"Erreur de connexion Redis: {e}")
            raise CacheConnectionError(f"Impossible de se connecter à Redis: {e}")

    async def disconnect(self) -> None:
        """Ferme la connexion Redis."""
        if self._client:
            await self._client.close()
            self._client = None
            logger.info("Connexion Redis fermée")

    async def get(self, key: str) -> Optional[Any]:
        """
        Récupère une valeur du cache.

        Args:
            key: Clé de la valeur

        Returns:
            Valeur désérialisée ou None
        """
        if not self._client:
            return None

        try:
            value = await self._client.get(key)
            if value is None:
                return None
            return json.loads(value)
        except (json.JSONDecodeError, Exception) as e:
            logger.warning(f"Erreur de lecture cache pour {key}: {e}")
            return None

    async def set(
        self,
        key: str,
        value: Any,
        ttl: Optional[int] = None
    ) -> None:
        """
        Stocke une valeur dans le cache.

        Args:
            key: Clé de la valeur
            value: Valeur à stocker (sera sérialisée en JSON)
            ttl: Durée de vie en secondes
        """
        if not self._client:
            return

        try:
            serialized = json.dumps(value)
            await self._client.set(
                key,
                serialized,
                ex=ttl or self.default_ttl,
            )
        except Exception as e:
            logger.warning(f"Erreur d'écriture cache pour {key}: {e}")

    async def delete(self, key: str) -> bool:
        """
        Supprime une valeur du cache.

        Args:
            key: Clé de la valeur

        Returns:
            True si supprimée
        """
        if not self._client:
            return False

        try:
            result = await self._client.delete(key)
            return result > 0
        except Exception as e:
            logger.warning(f"Erreur de suppression cache pour {key}: {e}")
            return False

    async def exists(self, key: str) -> bool:
        """
        Vérifie si une clé existe.

        Args:
            key: Clé à vérifier

        Returns:
            True si existe
        """
        if not self._client:
            return False

        try:
            return await self._client.exists(key) > 0
        except Exception:
            return False

    async def clear(self) -> None:
        """Vide tout le cache."""
        if not self._client:
            return

        try:
            await self._client.flushdb()
            logger.info("Cache Redis vidé")
        except Exception as e:
            logger.warning(f"Erreur lors du vidage du cache: {e}")

    async def is_connected(self) -> bool:
        """Vérifie si Redis est connecté."""
        if not self._client:
            return False

        try:
            await self._client.ping()
            return True
        except Exception:
            return False

    @property
    def backend_type(self) -> str:
        """Retourne 'redis'."""
        return "redis"

    async def set_hash(self, key: str, mapping: dict, ttl: Optional[int] = None) -> None:
        """
        Stocke un hash dans Redis.

        Args:
            key: Clé du hash
            mapping: Dictionnaire à stocker
            ttl: Durée de vie en secondes
        """
        if not self._client:
            return

        try:
            await self._client.hset(key, mapping=mapping)
            if ttl:
                await self._client.expire(key, ttl)
        except Exception as e:
            logger.warning(f"Erreur d'écriture hash pour {key}: {e}")

    async def get_hash(self, key: str) -> Optional[dict]:
        """
        Récupère un hash de Redis.

        Args:
            key: Clé du hash

        Returns:
            Dictionnaire ou None
        """
        if not self._client:
            return None

        try:
            return await self._client.hgetall(key)
        except Exception as e:
            logger.warning(f"Erreur de lecture hash pour {key}: {e}")
            return None

    async def scan_and_delete(self, pattern: str) -> int:
        """
        Supprime toutes les clés correspondant au pattern.

        Args:
            pattern: Pattern de recherche (ex: "acl:user_me:*")

        Returns:
            Nombre de clés supprimées
        """
        if not self._client:
            return 0

        deleted_count = 0
        try:
            async for key in self._client.scan_iter(match=pattern):
                await self._client.delete(key)
                deleted_count += 1

            if deleted_count > 0:
                logger.debug(f"Cache: {deleted_count} clés supprimées (pattern: {pattern})")

        except Exception as e:
            logger.warning(f"Erreur scan_and_delete pour {pattern}: {e}")

        return deleted_count



"""
Interface abstraite pour le cache.
"""

from abc import ABC, abstractmethod
from typing import Any, Optional


class CacheBackend(ABC):
    """
    Interface abstraite pour tous les backends de cache.

    Définit le contrat que chaque implémentation doit respecter.
    """

    @abstractmethod
    async def connect(self) -> None:
        """
        Établit la connexion au cache.

        Raises:
            CacheConnectionError: Si la connexion échoue
        """
        pass

    @abstractmethod
    async def disconnect(self) -> None:
        """Ferme la connexion au cache."""
        pass

    @abstractmethod
    async def get(self, key: str) -> Optional[Any]:
        """
        Récupère une valeur du cache.

        Args:
            key: Clé de la valeur

        Returns:
            Valeur ou None si non trouvée
        """
        pass

    @abstractmethod
    async def set(
        self,
        key: str,
        value: Any,
        ttl: Optional[int] = None
    ) -> None:
        """
        Stocke une valeur dans le cache.

        Args:
            key: Clé de la valeur
            value: Valeur à stocker
            ttl: Durée de vie en secondes (optionnel)
        """
        pass

    @abstractmethod
    async def delete(self, key: str) -> bool:
        """
        Supprime une valeur du cache.

        Args:
            key: Clé de la valeur

        Returns:
            True si supprimée, False si non trouvée
        """
        pass

    @abstractmethod
    async def exists(self, key: str) -> bool:
        """
        Vérifie si une clé existe dans le cache.

        Args:
            key: Clé à vérifier

        Returns:
            True si existe, False sinon
        """
        pass

    @abstractmethod
    async def clear(self) -> None:
        """Vide tout le cache."""
        pass

    @abstractmethod
    async def is_connected(self) -> bool:
        """
        Vérifie si le cache est connecté.

        Returns:
            True si connecté, False sinon
        """
        pass

    @property
    @abstractmethod
    def backend_type(self) -> str:
        """
        Retourne le type de backend.

        Returns:
            Type de backend (redis, memory)
        """
        pass



"""
Repository PostgreSQL pour l'authentification.

Supporte les modèles personnalisés avec colonnes supplémentaires.
"""

from typing import Optional, List, Type

from sqlalchemy import select, func
from sqlalchemy.exc import IntegrityError

from alak_acl.auth.application.interface.auth_repository import IAuthRepository
from alak_acl.auth.domain.entities.auth_user import AuthUser
from alak_acl.auth.infrastructure.mappers.auth_user_mapper import AuthUserMapper
from alak_acl.auth.infrastructure.models.sql_model import SQLAuthUserModel
from alak_acl.shared.database.postgresql import PostgreSQLDatabase
from alak_acl.shared.exceptions import UserAlreadyExistsError, UserNotFoundError
from alak_acl.shared.logging import logger


class PostgreSQLAuthRepository(IAuthRepository):
    """
    Implémentation du repository d'authentification pour PostgreSQL.

    Utilise SQLAlchemy 2.0 async avec asyncpg.
    Supporte les modèles utilisateur personnalisés avec colonnes supplémentaires.

    Attributes:
        db: Instance de connexion PostgreSQL
        model_class: Classe du modèle utilisateur (par défaut ou personnalisée)
        mapper: Instance du mapper (configuré pour le modèle personnalisé)

    Example:
        ```python
        # Avec modèle personnalisé
        from sqlalchemy import Column, String
        from alak_acl.auth.infrastructure.models import SQLAuthUserModel

        class CustomUserModel(SQLAuthUserModel):
            __tablename__ = "users"
            phone = Column(String(20), nullable=True)
            company = Column(String(100), nullable=True)

        repo = PostgreSQLAuthRepository(
            db=database,
            model_class=CustomUserModel,
        )
        ```
    """

    def __init__(
        self,
        db: PostgreSQLDatabase,
        model_class: Type[SQLAuthUserModel] = SQLAuthUserModel,
        mapper: Optional[AuthUserMapper] = None,
    ):
        """
        Initialise le repository.

        Args:
            db: Instance de PostgreSQLDatabase
            model_class: Classe du modèle utilisateur (par défaut ou personnalisée)
            mapper: Instance du mapper personnalisée (optionnel)
        """
        self._db = db
        self._model_class = model_class
        self._mapper = mapper or AuthUserMapper(sql_model_class=model_class)

    @property
    def model_class(self) -> Type[SQLAuthUserModel]:
        """Retourne la classe du modèle utilisé."""
        return self._model_class

    async def create_user(self, user: AuthUser) -> AuthUser:
        """
        Crée un nouvel utilisateur.

        Les champs personnalisés dans extra_fields sont automatiquement
        mappés vers les colonnes correspondantes du modèle.
        """
        model = self._mapper.to_sql_model(user, self._model_class)

        async with self._db.session() as session:
            try:
                session.add(model)
                await session.flush()
                await session.refresh(model)
                logger.debug(f"Utilisateur créé: {user.username}")
                return self._mapper.to_entity(model)
            except IntegrityError as e:
                logger.warning(f"Erreur d'intégrité lors de la création: {e}")
                raise UserAlreadyExistsError(
                    "__all__",
                    "Un utilisateur avec ce username ou email existe déjà"
                )

    async def get_by_id(self, user_id: str) -> Optional[AuthUser]:
        """Récupère un utilisateur par son ID (string UUID)."""
        async with self._db.session() as session:
            result = await session.execute(
                select(self._model_class).where(self._model_class.id == user_id)
            )
            model = result.scalar_one_or_none()

            if model:
                return self._mapper.to_entity(model)
            return None

    async def get_by_username(self, username: str) -> Optional[AuthUser]:
        """Récupère un utilisateur par son username."""
        async with self._db.session() as session:
            result = await session.execute(
                select(self._model_class).where(self._model_class.username == username)
            )
            model = result.scalar_one_or_none()

            if model:
                return self._mapper.to_entity(model)
            return None

    async def get_by_email(self, email: str) -> Optional[AuthUser]:
        """Récupère un utilisateur par son email."""
        async with self._db.session() as session:
            result = await session.execute(
                select(self._model_class).where(self._model_class.email == email)
            )
            model = result.scalar_one_or_none()

            if model:
                return self._mapper.to_entity(model)
            return None

    async def update_user(self, user: AuthUser) -> AuthUser:
        """
        Met à jour un utilisateur.

        Les champs personnalisés dans extra_fields sont automatiquement
        mis à jour dans les colonnes correspondantes.
        """
        async with self._db.session() as session:
            result = await session.execute(
                select(self._model_class).where(self._model_class.id == user.id)
            )
            model = result.scalar_one_or_none()

            if not model:
                raise UserNotFoundError("id", f"Utilisateur non trouvé: {user.id}")

            self._mapper.update_sql_model(model, user)
            await session.flush()
            await session.refresh(model)

            logger.debug(f"Utilisateur mis à jour: {user.username}")
            return self._mapper.to_entity(model)

    async def delete_user(self, user_id: str) -> bool:
        """Supprime un utilisateur par son ID (string UUID)."""
        async with self._db.session() as session:
            result = await session.execute(
                select(self._model_class).where(self._model_class.id == user_id)
            )
            model = result.scalar_one_or_none()

            if not model:
                return False

            await session.delete(model)
            logger.debug(f"Utilisateur supprimé: {user_id}")
            return True

    async def list_users(
        self,
        skip: int = 0,
        limit: int = 100,
        is_active: Optional[bool] = None,
    ) -> List[AuthUser]:
        """Liste les utilisateurs avec pagination."""
        async with self._db.session() as session:
            query = select(self._model_class)

            if is_active is not None:
                query = query.where(self._model_class.is_active == is_active)

            query = query.offset(skip).limit(limit).order_by(self._model_class.created_at.desc())

            result = await session.execute(query)
            models = result.scalars().all()

            return [self._mapper.to_entity(model) for model in models]

    async def count_users(self, is_active: Optional[bool] = None) -> int:
        """Compte le nombre d'utilisateurs."""
        async with self._db.session() as session:
            query = select(func.count(self._model_class.id))

            if is_active is not None:
                query = query.where(self._model_class.is_active == is_active)

            result = await session.execute(query)
            return result.scalar_one()

    async def username_exists(self, username: str) -> bool:
        """Vérifie si un username existe."""
        async with self._db.session() as session:
            result = await session.execute(
                select(func.count(self._model_class.id)).where(
                    self._model_class.username == username
                )
            )
            return result.scalar_one() > 0

    async def email_exists(self, email: str) -> bool:
        """Vérifie si un email existe."""
        async with self._db.session() as session:
            result = await session.execute(
                select(func.count(self._model_class.id)).where(
                    self._model_class.email == email
                )
            )
            return result.scalar_one() > 0

    async def find_by_extra_field(
        self,
        field_name: str,
        value: any,
    ) -> Optional[AuthUser]:
        """
        Recherche un utilisateur par un champ personnalisé.

        Args:
            field_name: Nom du champ personnalisé
            value: Valeur à rechercher

        Returns:
            Utilisateur trouvé ou None

        Raises:
            ValueError: Si le champ n'existe pas dans le modèle
        """
        if not hasattr(self._model_class, field_name):
            raise ValueError(f"Le champ '{field_name}' n'existe pas dans le modèle")

        async with self._db.session() as session:
            column = getattr(self._model_class, field_name)
            result = await session.execute(
                select(self._model_class).where(column == value)
            )
            model = result.scalar_one_or_none()

            if model:
                return self._mapper.to_entity(model)
            return None

    async def list_by_extra_field(
        self,
        field_name: str,
        value: any,
        skip: int = 0,
        limit: int = 100,
    ) -> List[AuthUser]:
        """
        Liste les utilisateurs par un champ personnalisé.

        Args:
            field_name: Nom du champ personnalisé
            value: Valeur à rechercher
            skip: Offset pour la pagination
            limit: Limite du nombre de résultats

        Returns:
            Liste d'utilisateurs correspondants
        """
        if not hasattr(self._model_class, field_name):
            raise ValueError(f"Le champ '{field_name}' n'existe pas dans le modèle")

        async with self._db.session() as session:
            column = getattr(self._model_class, field_name)
            query = (
                select(self._model_class)
                .where(column == value)
                .offset(skip)
                .limit(limit)
                .order_by(self._model_class.created_at.desc())
            )

            result = await session.execute(query)
            models = result.scalars().all()

            return [self._mapper.to_entity(model) for model in models]



"""
Interface abstraite pour les connexions base de données.
"""

from abc import ABC, abstractmethod
from typing import Any, Optional


class BaseDatabase(ABC):
    """
    Interface abstraite pour toutes les connexions base de données.

    Définit le contrat que chaque implémentation de DB doit respecter.
    """

    @abstractmethod
    async def connect(self) -> None:
        """
        Établit la connexion à la base de données.

        Raises:
            DatabaseConnectionError: Si la connexion échoue
        """
        pass

    @abstractmethod
    async def disconnect(self) -> None:
        """
        Ferme la connexion à la base de données.
        """
        pass

    @abstractmethod
    async def is_connected(self) -> bool:
        """
        Vérifie si la connexion est active.

        Returns:
            True si connecté, False sinon
        """
        pass

    @abstractmethod
    def get_session(self) -> Any:
        """
        Retourne une session/connexion pour les opérations.

        Returns:
            Session de base de données
        """
        pass

    @property
    @abstractmethod
    def db_type(self) -> str:
        """
        Retourne le type de base de données.

        Returns:
            Type de DB (mongodb, postgresql, mysql)
        """
        pass
